/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */



#include <limits.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "jomar3D.h"
#if USEGLEW
#  include <GL/glew.h>
#endif
#include <GL/glut.h>

#include "core.h"
#include "draw.h"
#include "display.h"
#include "interface.h"
#include "scores.h"
#include "sound.h"
#include "utils.h"

#ifndef LANGDIR
#  define LANGDIR DATADIR"/lang"
#endif

static char *preffile;


static float legb[4] = { -15., -5., 5., 25. };  // root angle for legs
static float legp[4] = { 0., 0., 0., 0. };   // position of the legs from the begin of the move


/* find if the lobster can go where the player wants */
int
tryplace (int actionhyp)
{
  int homposxt, homposyt, senshyp, dest;

  homposxt = homposx;
  homposyt = homposy;
  senshyp = terrain[homposx][homposy];
  switch (actionhyp)
    {
    case 2:                    // turn right
      senshyp++;
      break;
    case 3:                    // turn left
      senshyp--;
      break;
    }
  if (senshyp == 0)
    senshyp = 4;
  if (senshyp == 5)
    senshyp = 1;
  dest = 1;
  if (actionhyp == 4)
    dest = -2;
  switch (senshyp)
    {
    case 1:
      homposyt += dest;
      break;
    case 2:
      homposxt += dest;
      break;
    case 3:
      homposyt -= dest;
      break;
    case 4:
      homposxt -= dest;
      break;
    }

  if (homposxt == -1 || homposxt == T_X || homposyt == -1 || homposyt == T_Y)
    {
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
      if (actionhyp == lastaction && (actionhyp == 1 || actionhyp == 4))
        mixsound (WALL);
#endif
      godist = T_C;             // don't disturb next action by trying to eat if unpossible to move
      return 0;
    }
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
  if ((terrain[homposxt][homposyt] == T_WALL) &&
      (actionhyp == lastaction) && (actionhyp == 1 || actionhyp == 4))
    mixsound (WALL);
#endif
  if (terrain[homposxt][homposyt] != T_EMPTY)
    {
      godist = T_C;             // don't disturb next action by trying to eat if unpossible to move
      return 0;
    }

  terrain[homposx][homposy] = senshyp; // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
  return 1;
}


/* wait a little and retry when we follow a Martian */
void
startbehindmart (int dummy)
{
  (void) dummy;

  if (!tryplace (1))
    {
      glutTimerFunc (100, startbehindmart, 1);
    }
  else
    {
      SKeyDown (nextaction, 0, 0);
    }
}

/* find if the lobster can eat and do it */
int
tryeat (void)
{
  int dx = 0, dy = 0, i, pass, lastspeed = gotime; // first pass : try one case ahead, second, two cases

  glutSetCursor (GLUT_CURSOR_DESTROY);
  glutTimerFunc (2000, nocur, 0);
  for (pass = 1; pass < 3; pass++)
    {
      switch (terrain[homposx][homposy])
        {
        case T_LOBSTER_FORWARD:
          dy = pass;
          break;
        case T_LOBSTER_RIGHT:
          dx = pass;
          break;
        case T_LOBSTER_BACKWARD:
          dy = -pass;
          break;
        case T_LOBSTER_LEFT:
          dx = -pass;
          break;
        }
      if (homposx + dx < 0 || homposx + dx > T_X - 1 || homposy + dy < 0 ||
          homposy + dy > T_Y - 1)
        return 0;
#if (DEBUG)
      printf ("attack: %d \n", terrain[homposx + dx][homposy + dy]);
#endif
      if (gotime == 100.)
        return 0;               // don't allow trying to eat a second time

      if (terrain_martian (homposx + dx, homposy + dy))
        {
          terrain[homposx + dx][homposy + dy] = T_EMPTY;
          for (i = 0; i < nmartians; i++)
            {
              if (martposx[i] == homposx + dx && martposy[i] == homposy + dy
                  && martspeed[i] != 0.)
                {
                  martspeed[i] = 0.;
                  martleft--;
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
                  mixsound (PICKUP);
#endif
                  if (martleft == 0)
                    glutTimerFunc (4000, beginscores, 1);
                  break;
                }
            }
          break;
        }
    }

#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
  mixsound (TRYCATCH);
#endif
  gotime = 100.;
  godist = T_C * 2.;
  going = 0;                    //     change values to make a big jump and do it
  for (i = 0; i <= 3; i++)
    legp[i] = legp[i] + T_C * 1.5;
  i = 0;
  if (lastaction == 1 && nextaction != 0)
    i = GLUT_KEY_UP;
  openp = 1;
  SKeyDown (GLUT_KEY_UP, 1, 1);
  if (i == 0 || nextaction == 0)
    KeyUp (GLUT_KEY_UP, 1, 1);
  if (lastspeed == 99)
    gotime = 99.;
//nextaction = GLUT_KEY_UP;
//glutPostRedisplay();
//if (nextaction != 0) { SKeyDown(nextaction, 1, 1) ;}
  return 0;
}

/* move all the Martians in function of the place of the lobster */
void
activemart (void)
{
  int i, j, k;

  for (i = 0; i < nmartians; i++)
    {
#if (DEBUG)
      printf ("treating martian number %d\n", i);
#endif
    retry:
      if (martspeed[i] == 0.)
        continue;
      if (martact[i])
        {                       // moving
          glutIdleFunc (forcedraw);
          float rtime;          // remaining time

          rtime = marttime[i] + martspeed[i] - glutGet (GLUT_ELAPSED_TIME);
#if (DEBUG)
          printf ("time remaining to move rtime %f\n", rtime);
#endif
          if (rtime <= 0)
            {
              martact[i] = 0;
              martposxgo[i] = 0.;
              martposygo[i] = 0.;
              martangle[i] = martgoangle[i];
              goto retry;
            }
          martangle[i] = martangle[i] - (martangle[i] - martgoangle[i]) / 4.;
          if (martposxgo[i] > 0)
            martposxgo[i] = T_C / martspeed[i] * rtime;
          if (martposxgo[i] < 0)
            martposxgo[i] = -T_C / martspeed[i] * rtime;
          if (martposygo[i] > 0)
            martposygo[i] = T_C / martspeed[i] * rtime;
          if (martposygo[i] < 0)
            martposygo[i] = -T_C / martspeed[i] * rtime;
#if (DEBUG)
          printf ("angle      %f %f\n", martangle[i], martgoangle[i]);
          printf ("going pos  %f %f\n", martposxgo[i], martposygo[i]);
#endif
        }
      else
        {                       // looking around
          float tbasem;

          tbasem = glutGet (GLUT_ELAPSED_TIME);
          int lookx, looky;

          /*       searching for the lobster under the Martian      M
             x       \
             x x x      = martview
             x x x x x   /                  */
          for (j = -1; j >= -martview; j--)
            {
              for (k = -abs (j + 1); k <= abs (j + 1); k++)
                {
                  lookx = martposx[i] + k;
                  looky = martposy[i] + j;
                  if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
                    continue;
#if (DEBUG == 2)
                  printf ("looking at %d %d\n", lookx, looky);
#endif
                  if (terrain_lobster (lookx, looky))
                    {
                      if (terrain[martposx[i]][martposy[i] + 1] != T_EMPTY ||
                          martposy[i] == T_Y - 1)
                        {
                          if (terrain[martposx[i] + 1][martposy[i]] != T_EMPTY
                              || martposx[i] == T_X - 1)
                            {
                              if (terrain[martposx[i] - 1][martposy[i]] !=
                                  T_EMPTY || martposx[i] == 0)
                                goto impossible;
                              goto left;
                            }
                          goto right;
                        }
                    up:martgoangle[i] = 180.;
                      terrain[martposx[i]][martposy[i]] = T_EMPTY;
                      martposy[i] += 1;
                      terrain[martposx[i]][martposy[i]] = T_MARTIAN_FORWARD;
                      martposxgo[i] = 0.;
                      martposygo[i] = -T_C;
                      marttime[i] = tbasem;
                      martact[i] = 1;
                      goto startmove;
                    }
                }
            }
/*          searching for the lobster at the left of the Martian  x
                                                                  x x
                                                                  x x x M
                                                                  x x
                                                                  x                */
          for (j = -1; j >= -martview; j--)
            {
              for (k = -abs (j + 1); k <= abs (j + 1); k++)
                {
                  lookx = martposx[i] + j;
                  looky = martposy[i] + k;
                  if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
                    continue;
#if (DEBUG == 2)
                  printf ("looking at %d %d\n", lookx, looky);
#endif
                  if (terrain_lobster (lookx, looky))
                    {
                      if (terrain[martposx[i] + 1][martposy[i]] != T_EMPTY ||
                          martposx[i] == T_X - 1)
                        {
                          if (terrain[martposx[i]][martposy[i] + 1] != T_EMPTY
                              || martposy[i] == T_Y - 1)
                            {
                              if (terrain[martposx[i]][martposy[i] - 1] !=
                                  T_EMPTY || martposy[i] == 0)
                                goto impossible;
                              goto down;
                            }
                          goto up;
                        }
                    right:martgoangle[i] = 270.;
                      terrain[martposx[i]][martposy[i]] = T_EMPTY;
                      martposx[i] += 1;
                      terrain[martposx[i]][martposy[i]] = T_MARTIAN_FORWARD;
                      martposxgo[i] = -T_C;
                      marttime[i] = tbasem;
                      martact[i] = 1;
                      goto startmove;
                    }
                }
            }
          /*      searching for the lobster upwards the Martian  x x x x x  \
             x x x     = martview
             x      /
             M     /          */
          for (j = 1; j <= martview; j++)
            {
              for (k = -abs (j - 1); k <= abs (j - 1); k++)
                {
                  lookx = martposx[i] + k;
                  looky = martposy[i] + j;
                  if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
                    continue;
#if (DEBUG == 2)
                  printf ("looking at %d %d\n", lookx, looky);
#endif
                  if (terrain_lobster (lookx, looky))
                    {
                      if (terrain[martposx[i]][martposy[i] - 1] != T_EMPTY ||
                          martposy[i] == 0)
                        {
                          if (terrain[martposx[i] + 1][martposy[i]] != T_EMPTY
                              || martposx[i] == T_X - 1)
                            {
                              if (terrain[martposx[i] - 1][martposy[i]] !=
                                  T_EMPTY || martposx[i] == 0)
                                goto impossible;
                              goto left;
                            }
                          goto right;
                        }
                    down:martgoangle[i] = 0.;
                      terrain[martposx[i]][martposy[i]] = T_EMPTY;
                      martposy[i] -= 1;
                      terrain[martposx[i]][martposy[i]] = T_MARTIAN_FORWARD;
                      martposxgo[i] = 0.;
                      martposygo[i] = T_C;
                      marttime[i] = tbasem;
                      martact[i] = 1;
                      goto startmove;
                    }
                }
            }
/*          searching for the lobster at the right of the Martian       x
                                                                      x x
                                                                  M x x x
                                                                      x x
                                                                        x                     */
          for (j = 1; j <= martview; j++)
            {
              for (k = -abs (j - 1); k <= abs (j - 1); k++)
                {
                  lookx = martposx[i] + j;
                  looky = martposy[i] + k;
                  if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
                    continue;
#if (DEBUG == 2)
                  printf ("looking at %d %d %d\n", lookx, looky, j);
#endif
                  if (terrain_lobster (lookx, looky))
                    {
                      if (terrain[martposx[i] - 1][martposy[i]] != T_EMPTY ||
                          martposx[i] == 0)
                        {
                          if (terrain[martposx[i]][martposy[i] + 1] != T_EMPTY
                              || martposy[i] == T_Y - 1)
                            {
                              if (terrain[martposx[i]][martposy[i] - 1] !=
                                  T_EMPTY || martposy[i] == 0)
                                goto impossible;
                              goto down;
                            }
                          goto up;
                        }
                    left:martgoangle[i] = 90.;
                      terrain[martposx[i]][martposy[i]] = T_EMPTY;
                      martposx[i] -= 1;
                      terrain[martposx[i]][martposy[i]] = T_MARTIAN_FORWARD;
                      martposxgo[i] = T_C;
                      marttime[i] = tbasem;
                      martact[i] = 1;
                      goto startmove;
                    }
                }
            }
        impossible:i = i;
        }
    }
  return;
startmove:
  martsteps += 1;
  goto retry;
}

/* calculate the values from which the lobster's drawing depends */
void
calclobsterpos (void)
{
  int i, dest;
  float legs[4] = { 20., 20., 20., 20. }; // step
  float legd[4] = { -15., 5., 20., 45. }; // last position before putting the leg a step forward

  if (going)
    {
      float elapsed;            // time elapsed since beginning of the move

      elapsed = glutGet (GLUT_ELAPSED_TIME) - tbase;
//     if ( gotime == 1 && gone == 0 ) elapsed = 1;    // force the drawing if we try to eat even if it's too late
      if (elapsed > gotime)
        {
          for (i = 0; i <= 3; i++)
            legp[i] = legp[i] - gone;
          going = 0;
          gone = 0;
          anglehom = anglebasehom;
          if (action == 3)
            anglehom = -anglebasehom;
          dest = 1;
          if (action == 4)
            dest = -1;
          switch (terrain[homposx][homposy])
            {
            case 1:
              terrain[homposx][homposy] = T_EMPTY; // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              homposy += dest;
              terrain[homposx][homposy] = T_LOBSTER_FORWARD;   // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              break;
            case 2:
              terrain[homposx][homposy] = T_EMPTY; // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              homposx += dest;
              terrain[homposx][homposy] = T_LOBSTER_RIGHT;  // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              break;
            case 3:
              terrain[homposx][homposy] = T_EMPTY; // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              homposy -= dest;
              terrain[homposx][homposy] = T_LOBSTER_BACKWARD;  // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              break;
            case 4:
              terrain[homposx][homposy] = T_EMPTY; // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              homposx -= dest;
              terrain[homposx][homposy] = T_LOBSTER_LEFT;   // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
              break;
            default:
              printf ("error by orientation of the lobster(1) %d",
                      terrain[homposx][homposy]);
              exit (1);
              break;
            }

/*      don't rotate two times when boosted otherwise the game isn't no more playable */
          if ((lastaction == 3 || lastaction == 2) && gotime == 99.)
            {
              nextaction = 0;
            }

          lastaction = action;
          action = 0;
          godist = T_C;
          if (nextaction != 0)
            {
              SKeyDown (nextaction, 1, 1);
              calclobsterpos ();
            }
        }
      else
        {
          gone = elapsed / gotime * godist;
          if (action == 4)
            gone = -gone;
          for (i = 0; i <= 3; i++)
            {
              if (action != 4)
                if (lega[i] >= legd[i])
                  legp[i] = legp[i] + legs[i];
              if (action == 4)
                if (lega[i] <= legd[i] - atan (legs[i] / leglong) * raddeg)
                  legp[i] = legp[i] - legs[i];
              lega[i] = legb[i] + atan ((gone - legp[i]) / leglong) * raddeg;
            }
          rotdemhom = sin (elapsed * 2 * M_PI / gotime) * 20;
          if (action == 2)
            anglehom = -atan ((T_C - gone) / gone) * raddeg + anglebasehom;
          if (action == 3)
            anglehom = +atan ((T_C - gone) / gone) * raddeg - anglebasehom;
        }
    }
  else
    {
      glutIdleFunc (NULL);
      openp = 0;
      lastaction = 0;
#if (USESOUNDSDLMIXER)
      Mix_HaltChannel (WALK);
#endif
    }
}

/* place the wells */

void
poswells (void)
{
  int randx, randy, sens, ix, iy, i, j, k, k1, k2;

/*  loop on each case to set all empty (0) */
  for (i = 0; i < T_X; i++)
    for (j = 0; j < T_Y; j++)
      terrain[i][j] = T_EMPTY;

/* set the position of the lobster */
  terrain[homposx][homposy] = T_LOBSTER_FORWARD;   // sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x-
  terrain[homposx][homposy - 1] = T_LOCKED;  // don't put anything on the tail of the lobster

  for (i = 1; i <= walllong; i++)
    {
      for (j = 0; j < newall; j++)
        {
        deb:
          ix = iy = 0;

/*     IRANDX = POSITION EN X DU PREMIER MORCEAU DU MUR
       IRANDY = POSITION EN Y DU PREMIER MORCEAU DU MUR
       SENS   = SENS
                SI SENS = 1 VERS LA DROITE
                SI SENS = 2 VERS LE HAUT
                SI SENS = 3 VERS LA GAUCHE
                SI SENS = 4 VERS LE BAS                    */
          randx = (int) (myrand () * (T_X - 1) + 0.5) + 0;
          randy = (int) (myrand () * (T_Y - 1) + 0.5) + 0;
          sens = (int) (myrand () * (4 - 1) + 0.5) + 1;

          if (sens == 1)
            ix = 1;
          if (sens == 2)
            iy = 1;
          if (sens == 3)
            ix = -1;
          if (sens == 4)
            iy = -1;

#if (DEBUG)
          if (randx < 0)
            printf ("piiinponnn randx = %d\n", randx);
          if (randx >= T_X)
            printf ("piiinponnn randx = %d\n", randx);
          if (randy < 0)
            printf ("piiinponnn randy = %d\n", randx);
          if (randy >= T_Y)
            printf ("piiinponnn randy = %d\n", randx);
          printf ("randx, randy, walllength, wallnb %d %d %d %d\n", randx,
                  randy, i, j);
#endif
          if (randx + ix * i < 0)
            goto deb;
          if (randx + ix * i >= T_X)
            goto deb;
          if (randy + iy * i < 0)
            goto deb;
          if (randy + iy * i >= T_Y)
            goto deb;

//       EST-CE QUE LES CASES SONT INTERDITES ?
          for (k = 0; k < i; k++)
            if (terrain[randx + ix * (k)][randy + iy * (k)] != T_EMPTY)
              goto deb;

//       REMPLISSAGE DES CASES UTILISEES
          for (k = 0; k < i; k++)
            terrain[randx + ix * (k)][randy + iy * (k)] = T_WALL;

//       BLOCAGE DES CASES ADJACENTES
          for (k = 0; k < i; k++)
            for (k1 = -1; k1 < 2; k1++)
              for (k2 = -1; k2 < 2; k2++)
                if (randx + ix * k + k1 >= 0 && randx + ix * k + k1 < T_X &&
                    randy + iy * k + k2 >= 0 && randy + iy * k + k2 < T_Y &&
                    terrain[randx + ix * k + k1][randy + iy * k + k2] ==
                    T_EMPTY)
                  terrain[randx + ix * k + k1][randy + iy * k + k2] =
                    T_LOCKED;

//       REMPLISSAGE DU TABLEAU INDIQUANT LA PLACE DE CHAQUE MUR
          wells[i - 1][j][0] = randx;
          wells[i - 1][j][1] = randy;
          wells[i - 1][j][2] = sens;
#if (DEBUG)
          printf ("wall position: %d %d length, number %d %d\n",
                  wells[i - 1][j][0], wells[i - 1][j][1], i, j);
#endif
        }
    }

//    MISE A ZERO DES CASES VALANT -1(limitrophes) POUR QU'ELLES SOIENT VIDES
  for (k1 = 0; k1 < T_X; k1++)
    for (k2 = 0; k2 < T_Y; k2++)
      if (terrain[k1][k2] == T_LOCKED)
        terrain[k1][k2] = T_EMPTY;
}

/* place the Martians */
void
posmartians (void)
{
  int randx, randy, sens;       // random position and direction of the martian
  int i;

  for (i = 0; i < nmartians; i++)
    {
    occuped:
      randx = (int) (myrand () * (T_X - 1) + 0.5) + 0;
      randy = (int) (myrand () * (T_Y - 1) + 0.5) + 0;
      sens = (int) (myrand () * (4 - 1) + 0.5) + 1;   //  1: >y+       2: >x+       3: >y-       4: >x-
      if (terrain[randx][randy] == T_EMPTY)
        {
          terrain[randx][randy] = sens + 10; // deprecated but still used...
          martposx[i] = randx;
          martposy[i] = randy;
          martangle[i] = (sens - 1) * 90. + 180.;
        }
      else
        {
          goto occuped;
        }
    }

#if (DEBUG)
  int j;

  for (j = T_Y - 1; j >= 0; j--)
    {
      for (i = 0; i < T_X; i++)
        {
          printf ("%3d", terrain[i][j]);
        }
      printf ("\n");
    }
#endif
}


/* do the chronic work: calculate the frames done since last call and try a random to launch a sound sometimes */

void
docron (int dummy)
{
  (void) dummy;

  calculatefps ();

/* now, use a random to launch an ambient sound */
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
  if (myrand () > 0.97)
    {
      mixsound (CHOU);
    }
#endif

  glutTimerFunc (1000, docron, 0);  // don't forget to be called another time !

}

/* (re)initialize the variables for a new game */

void
initgame (void)
{
  int i;

  currentmenu = NOMENU;
  object = 0;
  homposx = 0;
  homposy = 1;
  poswells ();
  for (i = 0; i <= 3; i++)
    lega[i] = legb[i];
  martleft = nmartians;
  martsteps = 0;
  anglex = 0;
  angley = 0;
  posex = 0, posey = -T_C * 1.5 + 1.;
  anglehom = 0;
  going = gone = 0;
  gotime = 2;
  godist = T_C;
  boostrac = 0;
#if (USESOUNDSDLMIXER)
  if (playsound)
    Mix_HaltChannel (WALK);
#endif
/* set the speed of each Martian */
  for (i = 0; i < nmartians; i++)
    martspeed[i] = (0.2525 + i * 0.01) * 1000.;
#if (DEBUG)
  for (i = 0; i < nmartians; i++)
    printf ("speed set for mart %i %f\n", i, martspeed[i]);
#endif
  posmartians ();
  setgameattribs ();
  gamestarttime = glutGet (GLUT_ELAPSED_TIME);
  timeneeded = 0;
  glutPostRedisplay ();
}

/* automatically called routine on exit */
void
cleanquit (void)
{
  if (g_winmode)
    glutLeaveGameMode ();
#if USEGLEW
#  ifdef GL_ARB_vertex_buffer_object
  if (GLEW_ARB_vertex_buffer_object)
    glDeleteBuffersARB (1, &vbobuf);
#  endif
#endif
  printf ("See you soon.\n");
}

/* filter for the language files */
int
langfilter (const struct dirent *a)
{
  if (strstr (a->d_name + (strlen (a->d_name) - 4), ".jlg"))
    return 1;
  return 0;
}

/* load all available language */
void
loadlanguages (void)
{
/* commented lines are obsolete : I used a system command to list the directory */
  struct dirent **namelist;

//char * com = "dir -Nm *.jlg";
  char rep[MAXCMD];
  char *pos;

//int length=256;
  int i = 0, j = 0;
  int n;
  int nchar;
  FILE *file;
  char *ptr;

/*psystem (com, strlen(com), rep, &length );
  next: pos = strstr (rep+i, ".jlg");
        if (pos == NULL) goto done;
        nchar = pos-rep+4-i;
#if (DEBUG)
        printf ("nchar %d\n", nchar);
        printf ("language %d pos %d / %d\n", nlang, i, length);
#endif
        strncpy (lang[nlang], rep+i, nchar);
        *(lang[nlang]+nchar) = '\0';
        i = i + nchar + 2 ;
	
#if (DEBUG)
        puts(lang[nlang]);
#endif
        nlang++;
        if (nlang >= MAXLANG) goto done;
        if (i >= length) goto done;
        goto next;
  done:;
#if (DEBUG)
  printf ("nlang %d\n", nlang );
#endif     */

  nlang = myscandir (LANGDIR, &namelist, langfilter);
  if (nlang == -1)
    {
      printf
        ("Problem listing the language directory %s - you will probably experience problems...\n",
         LANGDIR);
      nlang = 0;
      return;
    }

  if (nlang > MAXLANG)
    nlang = MAXLANG;
  for (n = 0; n < nlang; n++)
    {
      strcpy (lang[n], namelist[n]->d_name);
#if (DEBUG)
      puts (lang[n]);
#endif
    }
#if (DEBUG)
  printf ("nlang %d\n", nlang);
#endif

/* load each language into memory */
  for (i = 0; i < nlang; i++)
    {
      ptr =
        malloc ((strlen (LANGDIR "/") + strlen (lang[i]) +
                 1) * sizeof (char));
      sprintf (ptr, "%s%s", LANGDIR "/", lang[i]);
      file = fopen (ptr, "r");
      if (file == NULL)
        {
          perror ("fopen for read");
          printf ("Error while opening a language: %s!\n", lang[i]);
          strcpy (lang[i], "unavailable");
          free (ptr);
          continue;
        }
      free (ptr);
      for (j = 0; j < NTEXT; j++)
        {
          if (fgets (rep, MAXCMD, file) == NULL)
            {
              perror ("fgets");
              printf ("Error while reading a language: %s!\n", lang[i]);
              strcpy (lang[i], "unavailable");
              break;
            }
          pos = strstr (rep, "\\");
          nchar = pos - rep;
          if (pos == NULL)
            nchar = strlen (rep) - 1;
#if (DEBUG)
          printf ("nchars %d\n", nchar);
#endif
          strncpy (text[i][j], rep, nchar);
          *(text[i][j] + nchar) = '\0';
#if (DEBUG)
          puts (text[i][j]);
#endif
        }
      fclose (file);
    }

// find if francais is existing     if not, make it
/*for (i=0;i<=nlang;i++) if (strcmp (lang[i], "francais.jlg") == 0) goto ok;
      if (nlang < MAXLANG) {
          nlang ++;
          lddeflang();      not working any more : file is no more created by the programme
      }
  ok:; */
// cut the file name's extension so the names are good for printing
  for (i = 0; i < nlang; i++)
    if (strcmp (lang[i], "unavailable"))
      lang[i][strstr (&lang[i][0], ".jlg") - &lang[i][0]] = '\0';
}

/* select the language to use reading jomar3Dpref */
void
selectlanguage (void)
{
  char rep[MAXCMD];
  FILE *file;
  int i;

  for (i = 0; i <= nlang; i++)
    if (strcmp (lang[i], "francais") == 0)
      uselangnb = i;
  file = fopen (preffile, "r");
  if (file == NULL)
    {
      perror ("fopen for read");
      printf ("Error opening %s.\n", preffile);
      return;
    }
  else
    {
      if (fgets (rep, MAXCMD, file) == NULL ||
          fgets (rep, MAXCMD, file) == NULL)
        {
          perror ("fgets");
          printf ("Error reading %s.\n", preffile);
          return;
        }
    }
  rep[strlen (rep) - 1] = '\0';
#if (DEBUG)
  puts (rep);
#endif
  for (i = 0; i <= nlang; i++)
    if (strcmp (lang[i], rep) == 0)
      uselangnb = i;
  fclose (file);
}

/* filter for the skin files */
int
skinfilter (const struct dirent *a)
{
//if (strstr (a->d_name, ".jsk") && a->d_type == DT_DIR) return 1; type of file unknown in the mingw32 so don't take care and hope...
  if (strstr (a->d_name + (strlen (a->d_name) - 4), ".jsk"))
    return 1;
  return 0;
}

/* find all skins in the directory */
void
findskins (void)
{
  struct dirent **namelist;
  int n;

  nskin = myscandir (SKINDIR, &namelist, skinfilter);
  if (nskin == -1)
    {
      printf
        ("Problem listing the skin directory %s - you will probably experience problems...\n",
         SKINDIR);
      nskin = 0;
      return;
    }

  if (nskin > MAXSKIN)
    nskin = MAXSKIN;
  for (n = 0; n < nskin; n++)
    {
      strcpy (skin[n], namelist[n]->d_name);
#if (DEBUG)
      puts (skin[n]);
#endif
    }
#if (DEBUG)
  printf ("nskin %d\n", nskin);
#endif
/* cut the file name's extension so the names are good for printing */
  for (n = 0; n < nskin; n++)
    skin[n][strstr (&skin[n][0], ".jsk") - &skin[n][0]] = '\0';
}

/* select the skin to use reading jomar3Dpref */
void
selectskin (void)
{
  char rep[MAXCMD];
  FILE *file;
  int i;

  for (i = 0; i <= nskin; i++)
    if (strcmp (skin[i], "summerskin") == 0)
      useskinnb = i;

  file = fopen (preffile, "r");
  if (file == NULL)
    {
      perror ("fopen for read");
      printf ("Error opening %s.\n", preffile);
      return;
    }
  else
    {
      if (fgets (rep, MAXCMD, file) == NULL ||
          fgets (rep, MAXCMD, file) == NULL ||
          fgets (rep, MAXCMD, file) == NULL ||
          fgets (rep, MAXCMD, file) == NULL ||
          fgets (rep, MAXCMD, file) == NULL ||
          fgets (rep, MAXCMD, file) == NULL)
        {
          perror ("fgets");
          printf ("Error reading %s.\n", preffile);
          return;
        }
    }
  rep[strlen (rep) - 1] = '\0';
#if (DEBUG)
  puts (rep);
#endif
  for (i = 0; i <= nskin; i++)
    if (strcmp (skin[i], rep) == 0)
      useskinnb = i;
  fclose (file);
}

/* write the options to use into jomar3Dpref */
void
saveoptions (void)
{
  FILE *file;
  int i;

  if (!(file = fopen (preffile, "r+")))
    perror ("fopen for update");
  if (!(file = fopen (preffile, "w")))
    {
      perror ("fopen for write");
      printf ("Cannot open %s... no save of the options!\n", preffile);
      return;
    }
  fprintf (file, "# language to use  possible: *.jlg (last run was \"");
  for (i = 0; i < nlang; i++)
    if (strcmp (lang[i], "unavailable"))
      fprintf (file, "%s, ", &lang[i][0]);
  fprintf (file, "\b\b\" available) #\n");
  if (fprintf (file, "%s\n", &lang[uselangnb][0]) < 0)
    {
      printf ("Cannot set language preference into jomar3Dpref\n");
      return;
    }

  fprintf (file, "# default resolution #\n\n");

  fprintf (file, "# look out (skin)  possible *.jsk (last run was \"");
  for (i = 0; i < nskin; i++)
    fprintf (file, "%s, ", &skin[i][0]);
  fprintf (file, "\b\b\" available) #\n");
  if (fprintf (file, "%s\n", &skin[useskinnb][0]) < 0)
    {
      printf ("Cannot set skin preference into jomar3Dpref\n");
      return;
    }
  fclose (file);
}

/* calculate the number of characters needed by the greatest integer number */
void
ncint (void)
{
  char buf[50];

/* find the number of characters per integer */
  ncharmaxint = sprintf (buf, "%lu", ULONG_MAX);
}

/* try to have real random by calling the list a few times in function of
 * the second the game is launched */
void
realrand (void)
{
  int i, now;

  now = time (NULL);
  for (i = 0; i < now - (int) (now / 1000) * 1000; i++)
    rand ();
}

void
printversion (void)
{
  printf ("Jomar3D compilation string (state DD/MM/YY): "VERSION"\n");
}

void
printhelp (void)
{
  printversion ();

  printf ("\nUsage: jomar3d [options]\n\n"
          "Options:\n"
          "\t-nosound                    Disable all sounds\n"
          "\t-version                    Print version and exit\n"
          "\t-help                       Print this help and exit\n" "\n");

  printf ("The X Window System specific options are: (same as GLUT's)\n"
          "  \n"
          "   -display DISPLAY \n"
          "           Specify the X server to connect to. If not specified, the value of the DISPLAY environment variable is used.         "
          "   \n"
          "   -geometry WxH+X+Y \n"
          "           Determines where the Jomar3D window should be created on the screen and its geometry.\n"
          "    \n"
          "   -iconic \n"
          "           Requests the window be created in an iconic state. \n"
          "     \n"
          "   -indirect \n"
          "           Force the use of indirect OpenGL rendering contexts. \n "
          "      \n"
          "   -direct \n"
          "           Force the use of direct OpenGL rendering contexts (not all GLX implementations support direct rendering  contexts)\n "
          "       \n"
          "   -gldebug\n"
          "           After processing callbacks and/or events, check if there are any OpenGL errors.\n"
          "         \n"
          "   -sync\n"
          "           Enable synchronous X protocol transactions.  This option makes it easier to track down potential X protocol errors.\n"
          "        \n");
}

/*  work on the command line arguments
 * \param argc main's argc
 * \param argv main's argv
 * \return 0 on successn 1 on success with a exit request, other on failed */
int
argswork (int argc, char **argv)
{
  int i;

#define CURARG (argv[i])        /* current arg */
#define ARGCMP(str) (!strcmp(CURARG, str))   /* compare current arg with str */

  for (i = 1; i < argc; i++)
    {
      if (ARGCMP ("-geometry"))
        {
          winuserdef = 1;
          i++;                  /* don't parse geometry */
        }
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
      else if (ARGCMP ("-nosound"))
        nosound = 1;
#endif
      else if (ARGCMP ("-version") || ARGCMP ("-V"))
        {
          printversion ();
          return 1;
        }
      else if (ARGCMP ("-help") || ARGCMP ("--help") || ARGCMP ("-h"))
        {
          printhelp ();
          return 1;
        }
#if (DEBUG)
      printf ("Argument number %d/%d: %s\n", i, argc, argv[i]);
#endif

    }

#undef ARGCMP
#undef CURARG

  return 0;
}

/* initialize and launch the game */
int
main (int argc, char **argv)
{
  int argrv;                    /* argument parser return value */
  char *ptr = getenv ("HOME");

  winuserdef = 0;
  g_winmode = 0;
  winwidth = 800;
  winheight = 600;
  ratio = 0.;
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
  nosound = 0;
#endif

  argrv = argswork (argc, argv);
  if (argrv != 0)
    {
      if (argrv == 1)
        return EXIT_SUCCESS;
      else
        return EXIT_FAILURE;
    }
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
  playsound = !nosound;
#endif

  ncint ();
  realrand ();

/* make a filename for the preference file */
  if (ptr == NULL)
    preffile = puts2p ("./.jomar3Dpref");
  else
    {
      preffile = malloc ((strlen (ptr) + 14) * sizeof (preffile));
      if (preffile != NULL)
        {
          strcpy (preffile, ptr);
          strcat (preffile, "/.jomar3Dpref");
        }
    }

  free (music);
  music = puts2p ("/");

/* start sdl initialization */
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
  /*nosound = !playsound;
     if (!nosound) {
     initializemusic();
     if (!nosound) SDL_CreateThread (loadallmusics, NULL);         // nosound could have changed
     } */
  launchmusic ();
#endif

/* load and select the current language */
  loadlanguages ();
  selectlanguage ();

/* locate all available skins and select the one to use now */
  findskins ();
  selectskin ();

/* set the function to call at exit */
  atexit (cleanquit);

//SDL_SetVideoMode( 800,  600, 0, SDL_FULLSCREEN);

  glutInit (&argc, argv);
  glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH |
                       GLUT_MULTISAMPLE);
//glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
//glutInitDisplayMode(GLUT_DOUBLE);
//glutInitDisplayString("double samples rgba");
#if (MYFREEGLUT)
  glutSetOption (GLUT_RENDERING_CONTEXT, GLUT_USE_CURRENT_CONTEXT);
  glutSetOption (GLUT_ACTION_ON_WINDOW_CLOSE, GLUT_ACTION_EXIT);
#endif
  if (!winuserdef)
    glutInitWindowSize (winwidth, winheight);
  else
    {
      winwidth = glutGet (GLUT_INIT_WINDOW_WIDTH);
      winheight = glutGet (GLUT_INIT_WINDOW_HEIGHT);
    }

  windowmanage ();
#if (MYFREEGLUT)
  loadtextures (1);
#endif
  quadObj = gluNewQuadric ();

#if USEGLEW
  GLenum err = glewInit ();

  if (GLEW_OK != err)
    {
      /* Problem: glewInit failed, something is seriously wrong. */
      fprintf (stderr, "Error: %s\n", glewGetErrorString (err));
    }
  fprintf (stdout, "Status: Using GLEW %s\n", glewGetString (GLEW_VERSION));
#endif

#ifdef GL_ARB_vertex_buffer_object
  /* get the supported extensions list */
  const unsigned char *exts = glGetString (GL_EXTENSIONS);

  /* look for the desired extension */
  if (strstr ((const char *) exts, "GL_ARB_vertex_buffer_object") == NULL)
    {
      /* extension not supported... :-( */
      fprintf (stderr,
               "Extension '%s' not supported, but continuing happily without...\n",
               "GL_ARB_vertex_buffer_object");
    }
  else
    {
      /* OK, extension supported */
      fprintf (stdout, "Extension '%s' is supported, using it...\n",
               "GL_ARB_vertex_buffer_object");
    }
#else
  fprintf (stdout, "Extension '%s' is not built in...\n",
           "GL_ARB_vertex_buffer_object");
#endif

//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

  glPushMatrix ();              /* dummy push so we can pop on model recalc */

  /* Setup floor plane for projected shadow calculations. */
  initshadow ();

  docron (0);                   // launch the cronic jobs (self-repetitive)
  currentmenu = NOMENU;
  changemenu (MP);

  glutMainLoop ();              /* launch all */

  return EXIT_SUCCESS;          /* ANSI C requires main to return int. */
}
