/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <limits.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "jomar3D.h"
#if (USEGLEW)
#  include <GL/glew.h>
#endif
#include <GL/glut.h>

#include "core.h"
#include "display.h"
#include "draw.h"
#include "interface.h"
#include "readtex.h"
#include "scores.h"
#include "sound.h"
#include "utils.h"

static void seescores (void);

static GLfloat pinceColor[] = { 1.0, 0.3, 0.3, 1.0 };
static GLfloat armColor[] = { 1.0, 0.5, 0.5, 1.0 };
static GLfloat tailColor[] = { 0.9, 0.2, 0.2, 1.0 };
static GLfloat bodyColor[] = { 0.9, 0.4, 0.4, 1.0 };
static GLfloat eyesColor[] = { 0.2, 0.2, 0.2, 1.0 };
static GLfloat monthColor[] = { 0.0, 0.5, 0.5, 1.0 };
static GLfloat legsColor[] = { 1.0, 0.5, 0.2, 1.0 };
static GLfloat mskinColor[] = { 0.2, 0.9, 0.3, 1.0 };
static GLfloat meyeColor[] = { 1.0, 0.8, 0.8, 1.0 };
static GLfloat soccerColor[] = { 0.9, 0.3, 0.3, 1.0 };
static GLfloat glassColor[] = { 0.8, 0.8, 1.0, 1.0 };
static GLfloat boosterColor[] = { 1.0, 0.8, 0.3, 1.0 };
static GLfloat pupilleColor[] = { 0.0, 0.0, 0.3, 1.0 };
static GLfloat fireColor[] = { 1.0, 1.0, 0.0, 0.5 };
static GLfloat ceilingColor[] = { 0.6, 0.6, 1.0, 1.0 };
static GLfloat lightColor[] = { 1.0, 1.0, 1.0, 1.0 };

//static GLfloat earthColor[]   = {0.0, 1.0, 0.0, 1.0};
//static GLfloat antColor[]     = {0.0, 0.0, 1.0, 0.5};
//static GLfloat fogColor[]     = {0.8, 0.8, 0.9, 1.0};

/* flag whether the drawing of the Martian should have boosters on or off */
enum
{ OFF, ON, SHADOW };

GLfloat lightPosition[] = { 0.0, 9000.0, 0.0, 1.0 };
GLfloat lightPositionmenu[] = { -100.0, 1000.0, -100.0, 1.0 };

GLfloat floorShadow[4][4];

GLuint texsid[15];              // groundid, extid, wall1id, wall2id, wall3id, wall4id, wall5id, wall6id, wall7id, wall8id, wall9id, wall10id;

                   // cloudid, logoid, menuid;

int textposition = 0;           // 0: left, 1: center, 2: right
int textmaxsize = 0;            // 0: as it must be, 1: fit to textmaxwidth
int textmaxwidth = 1500;        // 0: unactive  other: max width
int textback = 0;               // 0: unactive  other: draw a background behind the text
int entries[NMENU] = { 39, 5, 1, 1, 1, 1 };  // first: number of text lines before the first menu in the language files

                                        // then: number of text lines for each menu

static int measuretime = 0;
int frames;
int fps;

char *soundtxt;                 // text to print in the sound option cylinder

#define introduration 1000      // duration of the introduction
int introelapsed;               // time elapsed since beginning of introduction

static GLfloat wallsdata[] = {
// 1
  1.0, 0.0,
  T_C / 2., 0.0, 0.0,
  0.0, 0.0,
  -T_C / 2., 0.0, 0.0,
  0.0, 1.0,
  -T_C / 2. * .9, T_H * 2., 0.0,
  1.0, 1.0,
  T_C / 2. * 1.1, T_H * 2., 0.0,
  1.0, 0.0,
  0.0, 0.0, -T_C / 2.,
  0.0, 0.0,
  0.0, 0.0, T_C / 2.,
  0.0, 1.0,
  0.0, T_H * 2., T_C / 2. * .9,
  1.0, 1.0,
  0.0, T_H * 2., -T_C / 2. * 1.1,
// 2
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  2.0, 1.0,
  -T_C * 1.5, T_H, -T_C / 2.,
  2.0, 0.0,
  -T_C * 1.5, 0.0, -T_C / 2.,
  3.0, 1.0,
  -T_C * 1.5, T_H, T_C / 2.,
  3.0, 0.0,
  -T_C * 1.5, 0.0, T_C / 2.,
  5.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  5.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  6.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  6.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 3
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  3.0, 1.0,
  -T_C * 2.5, T_H, -T_C / 2.,
  3.0, 0.0,
  -T_C * 2.5, 0.0, -T_C / 2.,
  4.0, 1.0,
  -T_C * 2.5, T_H, T_C / 2.,
  4.0, 0.0,
  -T_C * 2.5, 0.0, T_C / 2.,
  7.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  7.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  8.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  8.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 4
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  4.0, 1.0,
  -T_C * 3.5, T_H, -T_C / 2.,
  4.0, 0.0,
  -T_C * 3.5, 0.0, -T_C / 2.,
  5.0, 1.0,
  -T_C * 3.5, T_H, T_C / 2.,
  5.0, 0.0,
  -T_C * 3.5, 0.0, T_C / 2.,
  9.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  9.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  10.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  10.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 5
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  5.0, 1.0,
  -T_C * 4.5, T_H, -T_C / 2.,
  5.0, 0.0,
  -T_C * 4.5, 0.0, -T_C / 2.,
  6.0, 1.0,
  -T_C * 4.5, T_H, T_C / 2.,
  6.0, 0.0,
  -T_C * 4.5, 0.0, T_C / 2.,
  11.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  11.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  12.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  12.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 6
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  6.0, 1.0,
  -T_C * 5.5, T_H, -T_C / 2.,
  6.0, 0.0,
  -T_C * 5.5, 0.0, -T_C / 2.,
  7.0, 1.0,
  -T_C * 5.5, T_H, T_C / 2.,
  7.0, 0.0,
  -T_C * 5.5, 0.0, T_C / 2.,
  13.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  13.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  14.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  14.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 7
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  7.0, 1.0,
  -T_C * 6.5, T_H, -T_C / 2.,
  7.0, 0.0,
  -T_C * 6.5, 0.0, -T_C / 2.,
  8.0, 1.0,
  -T_C * 6.5, T_H, T_C / 2.,
  8.0, 0.0,
  -T_C * 6.5, 0.0, T_C / 2.,
  15.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  15.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  16.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  16.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 8
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  8.0, 1.0,
  -T_C * 7.5, T_H, -T_C / 2.,
  8.0, 0.0,
  -T_C * 7.5, 0.0, -T_C / 2.,
  9.0, 1.0,
  -T_C * 7.5, T_H, T_C / 2.,
  9.0, 0.0,
  -T_C * 7.5, 0.0, T_C / 2.,
  17.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  17.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  18.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  18.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 9
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  9.0, 1.0,
  -T_C * 8.5, T_H, -T_C / 2.,
  9.0, 0.0,
  -T_C * 8.5, 0.0, -T_C / 2.,
  10.0, 1.0,
  -T_C * 8.5, T_H, T_C / 2.,
  10.0, 0.0,
  -T_C * 8.5, 0.0, T_C / 2.,
  19.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  19.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  20.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  20.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
// 10
  0.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  0.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.,
  10.0, 1.0,
  -T_C * 9.5, T_H, -T_C / 2.,
  10.0, 0.0,
  -T_C * 9.5, 0.0, -T_C / 2.,
  11.0, 1.0,
  -T_C * 9.5, T_H, T_C / 2.,
  11.0, 0.0,
  -T_C * 9.5, 0.0, T_C / 2.,
  21.0, 1.0,
  T_C / 2., T_H, T_C / 2.,
  21.0, 0.0,
  T_C / 2., 0.0, T_C / 2.,
  22.0, 1.0,
  T_C / 2., T_H, -T_C / 2.,
  22.0, 0.0,
  T_C / 2., 0.0, -T_C / 2.
};

void
forcedraw (void)
{
  glutPostRedisplay ();
}


/* calculate the frames drawn since last call */
void
calculatefps (void)
{
  int now = glutGet (GLUT_ELAPSED_TIME), elapsed = now - measuretime;

  fps = (int) (frames * 1000. / elapsed + 0.5);
  if (fps != 0)
    average = (average * 20. + (float) (fps)) / 21.;
#if (DEBUG)
  printf ("calcfps: %d %d %d %f, window number %d\n", elapsed, fps, frames,
          average, glutGetWindow ());
#endif
  frames = 0;
  measuretime = now;
}


/* very useful routine to output texts to the OpenGL window including scaling and background drawing */
void
output (GLfloat x, GLfloat y, GLfloat factx, GLfloat facty, char *text)
{
  char *p;
  GLfloat textColor[4];
  GLfloat width =
    glutStrokeLength (GLUT_STROKE_ROMAN, (const unsigned char *) text);
#if (DEBUG == 2)
  printf ("textwidth %f\n", width * factx);
#endif

  glPushMatrix ();
  glTranslatef (x, y, 0);
  if (width * factx > textmaxwidth || textmaxsize)
    factx = textmaxwidth / width;
  glScalef (factx, facty, 1.);
  if (textposition == 1)
    glTranslatef (-width / 2., 0, 0);
  if (textposition == 2)
    glTranslatef (-width, 0, 0);

/* draw a background for the text so it is much better readable (when flag on) */
  if (textback)
    {
      glGetFloatv (GL_CURRENT_COLOR, textColor);
      glColor4f (1. - textColor[0], 1. - textColor[1], 1. - textColor[2],
                 1. - textColor[3]);
      glRecti (-20, -40, width + 20, 130);
      glColor4fv (textColor);
    }

/* print the characters -- take care of some special character significations */
  for (p = text; *p; p++)
    {
      if (*p == '`' || *p == '' || *p == '^' || *p == '~' || *p == '"')
        {
          glPushMatrix ();
          glTranslatef (0.0, 55., 0.0);
          if (*p == '`')
            glTranslatef (0., -35.0, 0.);
          if (*p == '^')
            glScalef (0.7, 0.7, 1.);
          if (*p == '"')
            glTranslatef (12., -30.0, 0.);
          if (*p == '')
            {
              glRotatef (90., 0, 0, 1);
              glTranslatef (0., -120.0, 0.);
              glutStrokeCharacter (GLUT_STROKE_ROMAN, '`');
            }
          else
            {
              glutStrokeCharacter (GLUT_STROKE_ROMAN, *p);
            }
          glPopMatrix ();
        }
      else
        {
          glutStrokeCharacter (GLUT_STROKE_ROMAN, *p);
        }
    }
  glPopMatrix ();
}

/* Create a matrix that will project the desired shadow. */
void
shadowMatrix (GLfloat shadowMat[4][4], GLfloat groundplane[4],
              GLfloat lightpos[4])
{
  GLfloat dot;
  enum
  { X, Y, Z, W };

  /* Find dot product between light position vector and ground plane normal. */
  dot = groundplane[X] * lightpos[X] +
    groundplane[Y] * lightpos[Y] +
    groundplane[Z] * lightpos[Z] + groundplane[W] * lightpos[W];

  shadowMat[0][0] = dot - lightpos[X] * groundplane[X];
  shadowMat[1][0] = 0.f - lightpos[X] * groundplane[Y];
  shadowMat[2][0] = 0.f - lightpos[X] * groundplane[Z];
  shadowMat[3][0] = 0.f - lightpos[X] * groundplane[W];

  shadowMat[X][1] = 0.f - lightpos[Y] * groundplane[X];
  shadowMat[1][1] = dot - lightpos[Y] * groundplane[Y];
  shadowMat[2][1] = 0.f - lightpos[Y] * groundplane[Z];
  shadowMat[3][1] = 0.f - lightpos[Y] * groundplane[W];

  shadowMat[X][2] = 0.f - lightpos[Z] * groundplane[X];
  shadowMat[1][2] = 0.f - lightpos[Z] * groundplane[Y];
  shadowMat[2][2] = dot - lightpos[Z] * groundplane[Z];
  shadowMat[3][2] = 0.f - lightpos[Z] * groundplane[W];

  shadowMat[X][3] = 0.f - lightpos[W] * groundplane[X];
  shadowMat[1][3] = 0.f - lightpos[W] * groundplane[Y];
  shadowMat[2][3] = 0.f - lightpos[W] * groundplane[Z];
  shadowMat[3][3] = dot - lightpos[W] * groundplane[W];

}

/* Find the plane equation given 3 points. */
void
findPlane (GLfloat plane[4], GLfloat v0[3], GLfloat v1[3], GLfloat v2[3])
{
  GLfloat vec0[3], vec1[3];
  enum
  { A, B, C, D };
  enum
  { X, Y, Z, W };

  /* Need 2 vectors to find cross product. */
  vec0[X] = v1[X] - v0[X];
  vec0[Y] = v1[Y] - v0[Y];
  vec0[Z] = v1[Z] - v0[Z];

  vec1[X] = v2[X] - v0[X];
  vec1[Y] = v2[Y] - v0[Y];
  vec1[Z] = v2[Z] - v0[Z];

  /* find cross product to get A, B, and C of plane equation */
  plane[A] = vec0[Y] * vec1[Z] - vec0[Z] * vec1[Y];
  plane[B] = -(vec0[X] * vec1[Z] - vec0[Z] * vec1[X]);
  plane[C] = vec0[X] * vec1[Y] - vec0[Y] * vec1[X];

  plane[D] = -(plane[A] * v0[X] + plane[B] * v0[Y] + plane[C] * v0[Z]);
}

/* Setup floor plane for projected shadow calculations. */
void
initshadow (void)
{
  GLfloat floorPlane[4];
  GLfloat floorVertices[3][3] = {
    {10.0, 0.0, 10.0},
    {10.0, 0.0, -10.0},
    {-10.0, 0.0, -10.0},
  };
  findPlane (floorPlane, floorVertices[0], floorVertices[1],
             floorVertices[2]);
  shadowMatrix (floorShadow, floorPlane, lightPosition);
}

/* XXXXXXXXXXXXXXXXX INITIALIZATION AND DRAWING OF THE LOBSTER XXXXXXXXXXXXXXXXXXXXXX */
/* create drawing lists for the lobster */

void
makepince (void)
{
  glNewList (PINCEOUV, GL_COMPILE);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, pinceColor);
  glTranslatef (0.0, 0.0, -0.6);
  glRotatef (6, 0.0, 1.0, 0.0);
  gluCylinder (quadObj, 6.0, 9.0, 16.0, 15, 1);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluDisk (quadObj, 0.0, 6.0, 15, 1);
  glTranslatef (0.0, 0.0, 16.0);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  gluDisk (quadObj, 0.0, 9.0, 15, 1);
  glTranslatef (4.5, 0.0, -1.0);
  glRotatef (-00, 0.0, 1.0, 0.0);
  gluCylinder (quadObj, 4., 1.0, 25.0, 10, 1);
  glRotatef (00, 0.0, 1.0, 0.0);
  glTranslatef (-8.6, 0.0, -1.0);
  glRotatef (-20, 0.0, 1.0, 0.0);
  gluCylinder (quadObj, 3.0, 0.0, 20.0, 10, 1);
  glEndList ();

  glNewList (PINCECLO, GL_COMPILE);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, pinceColor);
  gluCylinder (quadObj, 6.0, 9.0, 16.0, 15, 1);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluDisk (quadObj, 0.0, 6.0, 15, 1);
  glTranslatef (0.0, 0.0, 16.0);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  gluDisk (quadObj, 0.0, 9.0, 15, 1);
  glTranslatef (4.5, 0.0, -1.0);
  glRotatef (-15, 0.0, 1.0, 0.0);
  gluCylinder (quadObj, 4.0, 1.0, 25.0, 10, 1);
  glRotatef (15, 0.0, 1.0, 0.0);
  glTranslatef (-9.9, 0.0, 1.0);
  gluCylinder (quadObj, 3.0, 0.0, 20.0, 10, 1);
  glEndList ();
}

void
makearms (void)
{
  glNewList (ARM, GL_COMPILE);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, armColor);
  glRotatef (28, 0.0, 1.0, 0.0);
  gluCylinder (quadObj, 7., 5., 15.0, 10, 1);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluDisk (quadObj, 0.0, 7., 10, 1);
  glTranslatef (0.0, 0.0, 15.0);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  gluDisk (quadObj, 0.0, 5., 10, 1);
  glTranslatef (0.0, 0.0, -1.8);
  glRotatef (-30, 0.0, 1.0, 0.0);
  gluCylinder (quadObj, 4.0, 3.5, 14.0, 10, 1);
  glTranslatef (0.0, 0.0, 14.0);
  glEndList ();
}

void
drawarms (void)
{
  glRotatef (10., 0.0, 1.0, 0.0);
  glCallList (ARM);

  glScalef (1.0, 0.6, 1.0);
  glRotatef (-10., 0.0, 1.0, 0.0);
  if (openp)
    {
      glCallList (PINCEOUV);
    }
  else
    {
      glTranslatef (0, 0, -0.5);
      glCallList (PINCECLO);
    }
}

void
makeant (void)
{
  int i;

  glNewList (ANT, GL_COMPILE);
  glBegin (GL_LINE_STRIP);
  for (i = 0; i < 60; i++)
    {
      glVertex3f (3 * sqrt (i), 0, -i);
    }
  glEnd ();
  glTranslatef (-0.2, 0, 0);
  glBegin (GL_LINE_STRIP);
  for (i = 0; i < 60; i++)
    {
      glVertex3f (3 * sqrt (i), 0, -i);
    }
  glEnd ();
  glEndList ();
}

void
makelegs (void)
{
  glNewList (LEG, GL_COMPILE);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, legsColor);
  glRotatef (90, 1.0, 0.0, 0.0);
  glRotatef (50, 0.0, 1.0, 0.0);
  gluCylinder (quadObj, 2., 0.5, 15.0, 5, 1);
  glTranslatef (0.0, 0.0, 14.9);
  glRotatef (-20, 0.0, 1.0, 0.0);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluDisk (quadObj, 0.0, 1.6, 5, 1);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  gluCylinder (quadObj, 1.6, 0.5, 13.0, 5, 1);
  glTranslatef (0.0, 0.0, 12.9);
  glRotatef (-10, 0.0, 1.0, 0.0);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluDisk (quadObj, 0.0, 1.4, 5, 1);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  gluCylinder (quadObj, 1.4, 0.2, 6.0, 5, 1);
  glEndList ();
}

void
maketail (void)
{
  glNewList (TAIL, GL_COMPILE);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, tailColor);
  if (boostrac == 0)
    {
      gluSphere (quadObj, 10.0, 20, 10);
      glRotatef (-5, 1.0, 0.0, 0.0);
      glRotatef (10, 0.0, 1.0, 0.0);
      glTranslatef (0, 0.0, -10.0);
    }
  if (boostrac < 2)
    {
      gluSphere (quadObj, 8.0, 18, 10);
      glRotatef (-5, 1.0, 0.0, 0.0);
      glRotatef (10, 0.0, 1.0, 0.0);
      glTranslatef (0, 0.0, -8.0);
    }
  if (boostrac < 3)
    {
      gluSphere (quadObj, 7.0, 15, 10);
      glTranslatef (-5.0, 0.0, -7.0);
      glRotatef (90, 1.0, 0.0, 0.0);
      glRotatef (10, 0.0, 1.0, 0.0);
    }
  if (boostrac == 3)
    {
      glTranslatef (00.0, 3.0, 00.0);
      gluSphere (quadObj, 3.0, 10, 10);
      glTranslatef (-5.0, 0.0, -3.0);
      glRotatef (90, 1.0, 0.0, 0.0);
      glRotatef (10, 0.0, 1.0, 0.0);
    }

/* commented lines draw the other side of the tail, which are never seen... */
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluDisk (quadObj, 0.0, 8.0, 20, 1);
//gluQuadricOrientation ( quadObj, GLU_OUTSIDE ) ;
//gluDisk(quadObj, 0.0,  8.0, 20, 1);
  glTranslatef (5.0, 0.0, 1.0);
  glRotatef (-10, 1.0, 1.0, 0.0);
  gluDisk (quadObj, 0.0, 7.0, 15, 1);
//gluQuadricOrientation ( quadObj, GLU_INSIDE ) ;
//gluDisk(quadObj, 0.0,  7.0, 15, 1);
//gluQuadricOrientation ( quadObj, GLU_OUTSIDE ) ;
  glTranslatef (5.0, 2.0, 1.0);
  glRotatef (-10, 1.0, 1.0, 0.0);
  gluDisk (quadObj, 0.0, 6.0, 15, 1);
//gluQuadricOrientation ( quadObj, GLU_INSIDE ) ;
//gluDisk(quadObj, 0.0,  6.0, 15, 1);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  glEndList ();
}

void
makebody (void)
{
  glNewList (BODY, GL_COMPILE);
  glTranslatef (0.0, 6.0, 0.0);
  glRotatef (-10, 1.0, 0.0, 0.0);
  /* eyes */
  glMaterialfv (GL_FRONT, GL_DIFFUSE, eyesColor);
  glTranslatef (3.5, 4.0, 0.0);
  gluSphere (quadObj, 2.0, 10, 10);
  glTranslatef (-7.0, 0.0, 0.0);
  gluSphere (quadObj, 2.0, 10, 10);
  glTranslatef (3.5, -4.0, 0.1);
  /* month */
  glMaterialfv (GL_FRONT, GL_DIFFUSE, monthColor);
  gluPartialDisk (quadObj, 0.0, 3.0, 15, 1, 45., 270.);
  glTranslatef (0.0, 0.0, -0.1);
  /* body */
  glMaterialfv (GL_FRONT, GL_DIFFUSE, bodyColor);
  gluDisk (quadObj, 0.0, 5.0, 15, 1);
  glTranslatef (0.0, 0.0, -7.0);
  gluCylinder (quadObj, 6.0, 5., 7.0, 15, 1);
  glTranslatef (0.0, 0.0, 2.7);
  glRotatef (10, 1.0, 0.0, 0.0);
  glRotatef (-25, 1.0, 0.0, 0.0);
  gluDisk (quadObj, 0.0, 7.0, 15, 1);
  glTranslatef (0.0, 0.0, -16.0);
  gluCylinder (quadObj, 11.8, 7., 16.0, 15, 1);
  glRotatef (25, 1.0, 0.0, 0.0);
  gluSphere (quadObj, 12.0, 20, 10);
  glTranslatef (0.0, 0.0, -12.0);
  gluSphere (quadObj, 11.0, 20, 10);
  glTranslatef (0.0, 0.0, -11.0);
  gluSphere (quadObj, 11.0, 20, 10);
  glEndList ();
}

/* draw the lobster using the lists below */
void
drawlobster (void)
{
  glTranslatef (0, 0, -29);
  glPushMatrix ();
  glTranslatef (04, 0, 0);
  glRotatef (rotdemhom / 2, 0.0, 1.0, 1.0);
  drawarms ();
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (-04, 0, 0);
  glRotatef (rotdemhom / 2, 0.0, 1.0, 1.0);
  glRotatef (180., 0.0, 0.0, 1.0);
  drawarms ();
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef (0, 0, 15);
  glCallList (BODY);
  glTranslatef (0, 0, -12);
  glRotatef (rotdemhom, 0.0, 1.0, 1.0);
  glCallList (TAIL);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef (6, 7, 14);
  glRotatef (rotdemhom / 1.7, 1.0, 1.0, 1.0);
  glCallList (ANT);
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (-6, 7, 14);
  glRotatef (180., 0.0, 0.0, 1.0);
  glRotatef (rotdemhom / 1.7, 1.0, 1.0, 1.0);
  glCallList (ANT);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef (8.0, 0, -6);
  glPushMatrix ();
  glRotatef (lega[0], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glTranslatef (0.0, 0.0, -8);
  glPushMatrix ();
  glRotatef (lega[1], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glTranslatef (0.0, 0.0, -8);
  glPushMatrix ();
  glRotatef (lega[2], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glTranslatef (0.0, 0.0, -8);
  glPushMatrix ();
  glRotatef (lega[3], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef (-8.0, 0, -6);
  glRotatef (180., 0.0, 1.0, 0.0);
  glPushMatrix ();
  glRotatef (-lega[0], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glTranslatef (0.0, 0.0, 8);
  glPushMatrix ();
  glRotatef (-lega[1], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glTranslatef (0.0, 0.0, 8);
  glPushMatrix ();
  glRotatef (-lega[2], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glTranslatef (0.0, 0.0, 8);
  glPushMatrix ();
  glRotatef (-lega[3], 0.0, 1.0, 0.0);
  glCallList (LEG);
  glPopMatrix ();
  glPopMatrix ();
}

/* XXXXXXXXXXXXXXXXX INITIALIZATION AND DRAWING OF THE MARTIAN  XXXXXXXXXXXXXXXXXXXXXX */
/* create the lists for the Martian */

void
makehead (void)
{
  glNewList (HEAD, GL_COMPILE);
  glTranslatef (0.0, 10.0, 6.0);
  glPushMatrix ();
  glMaterialfv (GL_FRONT, GL_DIFFUSE, mskinColor);
  glScalef (1.3, 1.0, 0.7);
  gluSphere (quadObj, 15.0, 10, 10);
  glPopMatrix ();

  glTranslatef (0.0, 24.0, 0.0);
  glPushMatrix ();
  glScalef (1.0, 0.6, 0.6);
  gluSphere (quadObj, 20.0, 10, 10);
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (-20 * .6, 6.0, -2.0);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, meyeColor);
  gluSphere (quadObj, 5.0, 5, 5);
  glTranslatef (0.0, 2.5, -4.5);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, pupilleColor);
  gluSphere (quadObj, 0.8, 4, 2);
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (0.0, 8.5, -4.0);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, meyeColor);
  gluSphere (quadObj, 5.5, 5, 5);
  glTranslatef (0.0, 2.5, -4.5);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, pupilleColor);
  gluSphere (quadObj, 1., 4, 2);
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (20 * .6, 6.0, -2.0);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, meyeColor);
  gluSphere (quadObj, 5.0, 5, 5);
  glTranslatef (0.0, 2.5, -4.5);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, pupilleColor);
  gluSphere (quadObj, 0.8, 4, 2);
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (6.0, 9.5, 4.0);
  glRotatef (-90., 1.0, 0.0, 0.0);
  glRotatef (10., 0.0, 1.0, 0.0);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, mskinColor);
  gluCylinder (quadObj, 1.0, 0.0, 15.0, 3, 1);
  glTranslatef (0.0, 0.0, 15.0);
  gluSphere (quadObj, 2.0, 2, 3);
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (-6.0, 9.5, 4.0);
  glRotatef (-90., 1.0, 0.0, 0.0);
  glRotatef (-10., 0.0, 1.0, 0.0);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, mskinColor);
  gluCylinder (quadObj, 1.0, 0.0, 15.0, 3, 1);
  glTranslatef (0.0, 0.0, 15.0);
  gluSphere (quadObj, 2.0, 2, 3);
  glPopMatrix ();
  glMaterialfv (GL_FRONT, GL_DIFFUSE, monthColor);
  glTranslatef (0.0, 0.0, -12.5);
  glRotatef (25., 1.0, 0.0, 0.0);
  glScalef (3.0, 1.0, 1.0);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluPartialDisk (quadObj, 0.0, 3.0, 10, 1, 55., 260.);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  glEndList ();
}

void
makemsoccer (void)
{
  glNewList (SOCCER, GL_COMPILE);
  glPushMatrix ();
  glMaterialfv (GL_FRONT, GL_DIFFUSE, soccerColor);
  glScalef (1.0, 0.4, 1.5);
  gluSphere (quadObj, 30.0, 8, 12);
  glPopMatrix ();
  glPushMatrix ();
  glTranslatef (9.0, 4.0, 35.0);
  glScalef (1.0, 0.8, 1.0);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, boosterColor);
  gluCylinder (quadObj, 3.5, 4.5, 15., 5, 1);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluCylinder (quadObj, 3.0, 4.0, 15., 5, 1);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  glTranslatef (-18.0, 0.0, 0.0);
  gluCylinder (quadObj, 3.5, 4.5, 15., 5, 1);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluCylinder (quadObj, 3.0, 4.0, 15., 5, 1);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  glPopMatrix ();
  glEndList ();
  glNewList (GLASS, GL_COMPILE);
  glPushMatrix ();
  glTranslatef (0.0, 8.0, -8.0);
  glRotatef (-90., 1.0, 0.0, 0.0);
  glScalef (1.0, 1.4, 1.0);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, glassColor);
  gluCylinder (quadObj, 20.0, 15.0, 9., 10, 1);
  gluQuadricOrientation (quadObj, GLU_INSIDE);
  gluCylinder (quadObj, 20.0, 15.0, 9., 10, 1);
  gluQuadricOrientation (quadObj, GLU_OUTSIDE);
  glPopMatrix ();
  glEndList ();

}

/* draw the Martian using the lists below */
void
drawmartian (int state)
{
  int npoints = 800, i;
  float randa, randr, randp, posz, angle = M_PI / 12., rayon = 20.;

  glPushMatrix ();
  glScalef (0.5, 0.5, 0.5);
  glCallList (SOCCER);
  if (state < 2)
    {                           /* rendering the shadow */
      glCallList (GLASS);
      glCallList (HEAD);
    }
  glPopMatrix ();
  if (state == 1 || state == 3)
    {
      glTranslatef (4.5, 2.0, 20.0);
      glDisable (GL_LIGHTING);
      glColor4f (fireColor[0], fireColor[1], fireColor[2], fireColor[3]);
      if (state > 1)
        glColor4f (0.0, 0.0, 0.0, 0.5);
      glBegin (GL_POINTS);
      for (i = 0; i < npoints; i++)
        {
          randa = (myrand () * 2. - 1.) * angle;
          randr = myrand () * rayon;
          posz = sin (randa) * randr;
          randp =
            (myrand () * 2. - 1.) * sqrt (pow (randr * sin (angle), 2) -
                                          posz * posz);
          glVertex3f (randp, posz, cos (randa) * randr);
        }
      glEnd ();
      glTranslatef (-9.0, 0.0, 00.0);
      glBegin (GL_POINTS);
      for (i = 0; i < npoints; i++)
        {
          randa = (myrand () * 2. - 1.) * angle;
          randr = myrand () * rayon;
          posz = sin (randa) * randr;
          randp =
            (myrand () * 2. - 1.) * sqrt (pow (randr * sin (angle), 2) -
                                          posz * posz);
          glVertex3f (randp, posz, cos (randa) * randr);
        }
      glEnd ();
      glEnable (GL_LIGHTING);
    }
}

/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX (end drawing initialization) */

/* draw the logo on top -- must be called the good matrix loaded */
void
playintro (void)
{
  float pos;

  if (introstarttime == 0)
    introstarttime = glutGet (GLUT_ELAPSED_TIME);

  introelapsed = glutGet (GLUT_ELAPSED_TIME) - introstarttime;
  if (introelapsed > introduration)
    return;

  glColor4f (0.0, 0.0, 0.0, 1.0);
  glEnable (GL_TEXTURE_2D);
  glBindTexture (GL_TEXTURE_2D, texsid[13]);
  if (introelapsed < introduration / 2.)
    {
      glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      pos = 1500. * (float) (introelapsed) / ((float) (introduration) / 2.);
      glBegin (GL_QUADS);
      glTexCoord2f (0.0, 0.0);
      glVertex2f (0.0, 0.0);
      glTexCoord2f (1.0, 0.0);
      glVertex2f (pos, 0.0);
      glTexCoord2f (1.0, 1.0);
      glVertex2f (pos, pos);
      glTexCoord2f (0.0, 1.0);
      glVertex2f (0.0, pos);
      glEnd ();
    }
  else
    {
      pos =
        1500. * ((float) (introelapsed) -
                 (float) (introduration) / 2.) / ((float) (introduration) /
                                                  2.);
      glBegin (GL_QUADS);
      glTexCoord2f (0.0, 0.0);
      glVertex2f (pos, pos);
      glTexCoord2f (1.0, 0.0);
      glVertex2f (1500, pos);
      glTexCoord2f (1.0, 1.0);
      glVertex2f (1500, 1500);
      glTexCoord2f (0.0, 1.0);
      glVertex2f (pos, 1500);
      glEnd ();
    }

  glDisable (GL_TEXTURE_2D);
  glutPostRedisplay ();
}

/* print all the texts needed by the various states */
void
gameprint (void)
{
  char buf[80];
  float timeelapsed = glutGet (GLUT_ELAPSED_TIME) - gamestarttime;

  glPushAttrib (GL_ENABLE_BIT);
  glDisable (GL_DEPTH_TEST);
  glDisable (GL_LIGHTING);
  glEnable (GL_LINE_SMOOTH);
  glEnable (GL_BLEND);
  glMatrixMode (GL_PROJECTION);
  glPushMatrix ();
  glLoadIdentity ();
  gluOrtho2D (0, 1500, 0, 1500);

  textposition = 1;
  textmaxwidth = 1500;
  textback = 1;
  if (martleft == 0)
    {
      if (timeneeded == 0.)
        timeneeded = glutGet (GLUT_ELAPSED_TIME) - gamestarttime;
      if (glutGet (GLUT_ELAPSED_TIME) - gamestarttime <
          (int) timeneeded + 3500)
        {
          glColor4f (1.0, 0.0, 0.0, 0.8);
          glLineWidth (5);
          output (750., 750., 1., 1., text[uselangnb][2]);
          glLineWidth (2);
          sprintf (buf, "%s %.3f %s", text[uselangnb][3], timeneeded / 1000.,
                   text[uselangnb][4]);
          output (750., 600., 0.4, 0.5, buf);
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
          mixsound (WON);
#endif
        }
      else
        {
          drawscores ();
        }
    }
  glLineWidth (2);
  glColor4f (1.0, 1.0, 1.0, 0.5);
  if (martleft == 1)
    sprintf (buf, "%s", text[uselangnb][1]);
  else
    sprintf (buf, "%s %d", text[uselangnb][0], martleft);
  if (currentmenu != NOMENU)
    sprintf (buf, "This game    was created and compiled    under Linux.");
  output (1500. / 2., 10, 1.15, 0.3, buf);
  textposition = 2;
  if (currentmenu == NOMENU)
    {
      sprintf (buf, "%s %d - %d", text[uselangnb][5], fps,
               (int) (average + 0.5));
      output (1493, 59, 0.32, 0.24, buf);
    }

  glLineWidth (1.8);
  glColor4f (0.0, 0.0, 0.0, 0.5);
  if (currentmenu != NOMENU)
    {
      textback = 0;
      textposition = 1;
      output (750., 1470., 1.15, 0.2, "c Studio Chalmion");
      output (750. - 638., 1470. - 08., 1.15, 0.3, "O");
      sprintf (buf, "jomar3D %s", VERSION);
      glColor4f (1.0, 0.0, 0.0, 0.5);
      textmaxwidth = 900;
      output (750., 1400., 1.15, 0.3, buf);
      textmaxwidth = 1500;
    }
  else
    {
      textposition = 2;
      sprintf (buf, "%lu", martsteps);
      output (1491, 1461, 0.5, 0.3, buf);
      sprintf (buf, "%.3f", timeelapsed / 1000.);
      output (1491, 1410, 0.5, 0.3, buf);
      textposition = 0;
      output (10, 1475, 0.5, 0.2, music);
      textback = 0;
    }

  if (currentmenu == NOMENU)
    {
      glPushMatrix ();
      glLoadIdentity ();
      gluOrtho2D (0, 1200 * ratio, 0, 1200);

      glColor4f (0.0, 1.0, 0.0, 1.0);
      glRecti (homposx * 7, 40 + homposy * 7, (homposx + 1) * 7,
               40 + (homposy + 1) * 7);
      glCallList (SMALLMAP);
      glPopMatrix ();
    }

  playintro ();

  glMatrixMode (GL_PROJECTION);
  glPopMatrix ();
  glPopAttrib ();
  glMatrixMode (GL_MODELVIEW);
}

/* create the list to call for the small map */
void
createsmap (void)
{
  int x, y;

  glNewList (SMALLMAP, GL_COMPILE);

  glColor4f (1.0, 1.0, 1.0, 0.5);
  glRecti (0, 40, 7 * T_X, 7 * T_Y + 40);

  glColor4f (1.0, 0.0, 0.0, 0.1);

  for (x = 0; x < T_X; x++)
    {
      for (y = 0; y < T_Y; y++)
        {
          if (terrain[x][y] == T_WALL)
            {
              glRecti (x * 7, 40 + y * 7, (x + 1) * 7, 40 + (y + 1) * 7);
            }
        }
    }

  glEndList ();
}


/* draw the rule menu */

void
drawrules (void)
{
  char buf[80];

  textmaxwidth = 1400.;
  textposition = 1;
  glColor4f (0.0, 0.2, 0.2, 1.0);
  output (750., 700., 0.4, 0.3, text[uselangnb][6]);
  textposition = 0;
  output (40., 630., 0.4, 0.30, text[uselangnb][7]);
  output (60., 580., 0.4, 0.30, text[uselangnb][8]);
  output (40., 510., 0.4, 0.30, text[uselangnb][9]);
  textmaxsize = 1;
  output (60., 460., 0.4, 0.30, text[uselangnb][10]);
  output (60., 410., 0.4, 0.30, text[uselangnb][11]);
  output (60., 360., 0.4, 0.30, text[uselangnb][12]);
  sprintf (buf, "%lu %s", LONG_MAX, text[uselangnb][13]);
  output (60., 310., 0.4, 0.30, buf);
  textmaxsize = 0;
  output (40., 240., 0.4, 0.30, text[uselangnb][14]);
  textmaxsize = 1;
  output (60., 190., 0.4, 0.30, text[uselangnb][15]);
  textposition = 1;
  output (750., 095., 0.4, 0.30, text[uselangnb][25]);
  textposition = 0;
  textmaxsize = 0;
}

/* draw the finds menu */

void
drawfinds (void)
{
  textmaxwidth = 1400.;
  textmaxsize = 0;
  textposition = 0;
  glColor4f (0.0, 0.2, 0.2, 1.0);
  output (40., 700., 0.4, 0.30, text[uselangnb][18]);
  output (40., 650., 0.4, 0.30, text[uselangnb][19]);
  output (40., 580., 0.4, 0.30, text[uselangnb][20]);
  output (40., 530., 0.4, 0.30, text[uselangnb][21]);
  output (40., 460., 0.4, 0.30, text[uselangnb][22]);
  output (40., 390., 0.4, 0.30, text[uselangnb][23]);
  output (40., 340., 0.4, 0.30, text[uselangnb][24]);
  output (40., 270., 0.4, 0.30, text[uselangnb][25]);
  output (40., 200., 0.4, 0.30, text[uselangnb][26]);
  textmaxsize = 1;
  textposition = 1;
  output (750., 100., 0.4, 0.30, text[uselangnb][27]);
  textmaxsize = 0;
}

/* draw the right text for the current menu */
void
drawtext (void)
{

  glPushAttrib (GL_ENABLE_BIT);
  glDisable (GL_DEPTH_TEST);
  glDisable (GL_LIGHTING);
  glEnable (GL_LINE_SMOOTH);
  glEnable (GL_BLEND);
  glMatrixMode (GL_PROJECTION);
  glPushMatrix ();
  glLoadIdentity ();
  gluOrtho2D (0, 1500, 0, 1500);


  glColor4f (1.0, 1.0, 1.0, 0.7);
  glRecti (20, 20, 1480, 750);

  if (currentmenu == RULES)
    drawrules ();
  if (currentmenu == SCORES)
    seescores ();
  if (currentmenu == FINDS)
    drawfinds ();

  glMatrixMode (GL_PROJECTION);
  glPopMatrix ();
  glPopAttrib ();
  glMatrixMode (GL_MODELVIEW);
}

int cursor[8] = { GLUT_CURSOR_TOP_SIDE,
  GLUT_CURSOR_TOP_RIGHT_CORNER,
  GLUT_CURSOR_RIGHT_SIDE,
  GLUT_CURSOR_BOTTOM_RIGHT_CORNER,
  GLUT_CURSOR_BOTTOM_SIDE,
  GLUT_CURSOR_BOTTOM_LEFT_CORNER,
  GLUT_CURSOR_LEFT_SIDE,
  GLUT_CURSOR_TOP_LEFT_CORNER
};

/* (re)draw all the window */
void
redraw (void)
{
  float shift = 0, anglexv = anglex, gonev;
  int length, number;

/* don't clear the screen, it's very long */
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//glClear(GL_DEPTH_BUFFER_BIT);

/* Set the view */
  glPushMatrix ();
  calclobsterpos ();
  gonev = gone;

/* change a few values if rotating  */
  if (action == 2)
    {
      anglexv = anglex - anglehom + anglebasehom;
      shift = -gone * ((-posey + posex) / T_C) - posey + posex;
      gonev = gone * ((posey + (T_C - 2) + posex) / T_C) - posey - posex;
    }
  if (action == 3)
    {
      anglexv = anglex - anglehom - anglebasehom;
      shift = gone * ((-posey - posex) / T_C) + posey + posex;
      gonev = -gone * ((-posey - (T_C - 2) + posex) / T_C) - posey + posex;
    }
  if (godist == T_C * 2.)
    gonev = gonev / 2.;

  switch (terrain[homposx][homposy])
    {                           // sens 1 > y+  sens 2 > x+  sens 3 > y-  sens 4 > x-
    case T_LOBSTER_FORWARD:
      gluLookAt (-homposx * T_C + posex - shift, 50.0, homposy * T_C + posey + gonev,  /* eye   */
                 -homposx * T_C + 200.0 * sin (anglexv * degrad) + posex, 20.0 + 200.0 * tan (angley * degrad), homposy * T_C + gonev + 200.0 * cos (anglexv * degrad) + posey,  /* center */
                 0.0, 1.0, 0.); /* up is in positive Y direction */
      break;
    case T_LOBSTER_RIGHT:
      gluLookAt (-homposx * T_C - posey - gonev, 50.0, homposy * T_C + posex - shift,  /* eye   */
                 -homposx * T_C - gonev + 200.0 * sin (anglexv * degrad - M_PI / 2.) - posey, 20.0 + 200.0 * tan (angley * degrad), homposy * T_C + 200.0 * cos (anglexv * degrad - M_PI / 2.) + posex,  /* center */
                 0.0, 1.0, 0.); /* up is in positive Y direction */
      break;
    case T_LOBSTER_BACKWARD:
      gluLookAt (-homposx * T_C - posex + shift, 50.0, homposy * T_C - posey - gonev,  /* eye   */
                 -homposx * T_C + 200.0 * sin (anglexv * degrad - M_PI) - posex, 20.0 + 200.0 * tan (angley * degrad), homposy * T_C - gonev + 200.0 * cos (anglexv * degrad - M_PI) - posey,   /* center */
                 0.0, 1.0, 0.); /* up is in positive Y direction */
      break;
    case T_LOBSTER_LEFT:
      gluLookAt (-homposx * T_C + posey + gonev, 50.0, homposy * T_C - posex + shift,  /* eye   */
                 -homposx * T_C + gonev + 200.0 * sin (anglexv * degrad + M_PI / 2.) + posey, 20.0 + 200.0 * tan (angley * degrad), homposy * T_C + 200.0 * cos (anglexv * degrad + M_PI / 2.) - posex,  /* center */
                 0.0, 1.0, 0.); /* up is in positive Y direction */
      break;
    default:
      printf ("error by orientation of the lobster(2) %d",
              terrain[homposx][homposy]);
      exit (1);
      break;
    }

  glLightfv (GL_LIGHT0, GL_POSITION, lightPosition);

/*  draw the world at the beginning so we can have good shadows on the bottom */

  glDisable (GL_LIGHTING);
  glDepthFunc (GL_ALWAYS);

  glCallList (TOPBOT);

  glDepthMask (GL_FALSE);
  glPushMatrix ();
  glTranslatef (-homposx * T_C, 0.0, homposy * T_C);
  glRotatef (-anglehom, 0.0, 1.0, 0.0);
  glTranslatef (0., 0., gone);
  glTranslatef (0.0, 3000., 0.0);
  glColor4f (1., 1., 0., 1.);
  glRotatef (90, 1.0, 0.0, 0.0);
  gluDisk (quadObj, 0, 150., 20, 1);
  glPopMatrix ();

  glEnable (GL_TEXTURE_2D);
  glCallList (CLOUDS);
  glDepthMask (GL_TRUE);
  glCallList (wallmax);
  glDisable (GL_TEXTURE_2D);

  glDepthFunc (GL_LESS);
  glEnable (GL_LIGHTING);

/* Draw the lobster  */
  glPushMatrix ();
  glTranslatef (-homposx * T_C, 0.0, homposy * T_C);
  glRotatef (-anglehom, 0.0, 1.0, 0.0);

  /*    shadow    */
  glPushMatrix ();
  glTranslatef (0., 1.5, gone);

  /* Render 50% black shadow color on top of whatever the floor appearance is. */
  glEnable (GL_BLEND);
  glDisable (GL_LIGHTING);      /* Force the 50% black. */
  glColor4f (0.0, 0.0, 0.0, 0.5);

  /* Project the shadow. */
  glDepthRange (0.999, 0.9990001);
  glDepthFunc (GL_GREATER);
  glMultMatrixf ((GLfloat *) floorShadow);
  drawlobster ();
  glDepthFunc (GL_LESS);
  glDepthRange (0., 1.);

  glDisable (GL_BLEND);
  glEnable (GL_LIGHTING);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef (0., leglong + rotdemhom / 10., gone);
  glShadeModel (GL_SMOOTH);
  drawlobster ();
  glShadeModel (GL_FLAT);
  glPopMatrix ();
  glPopMatrix ();

/* Active the Martians (look around or move) */
  activemart ();
/* Draw the Martians  */
  int i;

  for (i = 0; i < nmartians; i++)
    {
      if (martspeed[i] == 0.)
        continue;
      glPushMatrix ();
      glTranslatef (-martposx[i] * T_C - martposxgo[i], 20.0,
                    martposy[i] * T_C + martposygo[i]);
      glRotatef (-martangle[i], 0., 1., 0.);
      glPushMatrix ();
      glShadeModel (GL_SMOOTH);
      if (martact[i])
        {
          drawmartian (ON);
        }
      else
        {
          drawmartian (OFF);
        }
      glShadeModel (GL_FLAT);
      glPopMatrix ();
      glPushMatrix ();
      glTranslatef (0., -18.0, 0.0);

      /* Render 50% black shadow color on top of whatever the floor appearance is. */
      glEnable (GL_BLEND);
      glDisable (GL_LIGHTING);  /* Force the 50% black. */
      glColor4f (0.0, 0.0, 0.0, 0.5);

      /* Project the shadow. */
      glMultMatrixf ((GLfloat *) floorShadow);
      if (martact[i])
        {
          drawmartian (ON | SHADOW);
        }
      else
        {
          drawmartian (OFF | SHADOW);
        }

      glDisable (GL_BLEND);
      glEnable (GL_LIGHTING);
      glPopMatrix ();
      glPopMatrix ();
    }

/*  draw the wells at the end so we can see if Martians are behind the wells */
  glDisable (GL_LIGHTING);

  glEnable (GL_TEXTURE_2D);
  glEnable (GL_ALPHA_TEST);
  glAlphaFunc (GL_GREATER, 0.5);

/* loop on each wall */
  for (length = walllong - 1; length >= 0; length--)
    for (number = 0; number < newall; number++)
      {
        glPushMatrix ();
        glTranslatef (-wells[length][number][0] * T_C, 0.,
                      wells[length][number][1] * T_C);
        glRotatef ((wells[length][number][2] - 1) * 90., 0.0, 1.0, 0.0);
        glBindTexture (GL_TEXTURE_2D, texsid[length + 2]);
        if (length == 0)
          {
            glEnable (GL_BLEND);
            glAlphaFunc (GL_GREATER, 0.0);
            glDisable (GL_CULL_FACE);
            glDepthMask (GL_FALSE);
            glDrawArrays (GL_QUADS, 0, 8);
            glDepthMask (GL_TRUE);
            glDisable (GL_BLEND);
            glEnable (GL_CULL_FACE);
          }
        else
          {
            glDrawArrays (GL_QUAD_STRIP, length * 10 - 2, 10);
          }
        glPopMatrix ();
      };
  glDisable (GL_ALPHA_TEST);
  glDisable (GL_TEXTURE_2D);

  glPopMatrix ();
  gameprint ();
  glEnable (GL_LIGHTING);


/* draw the menu after all so it does not change in function of what is behind
 * only if we are in a menu and the intro is finished  */
  if (currentmenu != NOMENU && introelapsed > introduration)
    {
      glLightfv (GL_LIGHT0, GL_POSITION, lightPositionmenu);

      glDepthFunc (GL_ALWAYS);
      glDepthMask (GL_FALSE);
      glCallList (MENU);
      if (currentmenu == RULES || currentmenu == FINDS ||
          currentmenu == SCORES)
        drawtext ();
      glDepthMask (GL_TRUE);
      glDepthFunc (GL_LESS);
    }

  glutSwapBuffers ();
  frames++;

#if (DEBUG)
  glutReportErrors ();
#endif
}

/* create the needed lists for the menu */

void
createmenulists (int listnb)
{                               // arg: list number to do (0 = all)
  int i;
  float cylssize = 60., cylcol[4];
  static float menuwidth;       // Width of each text cylinder for the current menu.
  int firsttext = 0;            // Position of the first text of this menu.

  i = listnb;
  if (listnb != 0)
    goto remake;

/* cylinders initialization */
  for (i = 0; i < NCYL; i++)
    {
      if (cyls[i][0] != -90.)
        cyls[i][0] = 0.;
      cyls[i][1] = cyls[i][2] = cyls[i][3] = 1.;
      cylstext[i] = NULL;
    }
  for (i = 0; i < currentmenu; i++)
    firsttext += entries[i];
#if (DEBUG)
  printf ("Menu first text pos: %d, is %s\n", firsttext,
          text[uselangnb][firsttext]);
#endif
  for (i = 0; i < entries[currentmenu]; i++)
    cylstext[i] = text[uselangnb][firsttext + i];
  if (currentmenu == OPTIONS)
    {
      cylstext[0] = lang[uselangnb];
      cylstext[1] = skin[useskinnb];
      cylstext[2] = NULL;
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
      if (playsound)
#  if (USESOUNDSDL)
        soundtxt = puts2p (MUSICDIR "/*.wav & gwavs");
#  elif (USESOUNDSDLMIXER)
        soundtxt = puts2p (MUSICDIR "/*.ogg, *.wav, *.voc & gwavs");
#  endif
      else
        soundtxt = puts2p ("/");
      if (!nosound)
        cylstext[2] = soundtxt;
#endif
      cylstext[3] = text[uselangnb][firsttext];
    }
  if (currentmenu == SCORES)
    {
      for (i = 0; i < 9; i++)
        {
          if (strlen (names[i]) == 0)
            break;
          cylstext[i] = names[i];
        }
      cylstext[i] = text[uselangnb][firsttext];
    }
  for (i = 0, menuwidth = 0.; i < NCYL - 1; i++)
    if (cylstext[i] != NULL)
      menuwidth =
        max (glutStrokeLength
             (GLUT_STROKE_ROMAN, (const unsigned char *) cylstext[i]),
             menuwidth);
  menuwidth *= 1.1;             // be a little brighter than exact width
  if (menuwidth > 2500)
    menuwidth = 2500;
#if (DEBUG)
  printf ("Menu width: %f\n", menuwidth);
#endif


/* create the list to call to draw the current menu */
  glNewList (MENU, GL_COMPILE);
  glShadeModel (GL_SMOOTH);
  glLineWidth (1.5);
  glPushMatrix ();
  glTranslatef (00., 00., -1000. - menuwidth / 10.);
  glTranslatef (0., NCYL * cylssize * 1.25, 0.);
  glPushMatrix ();
  glMaterialfv (GL_FRONT, GL_DIFFUSE, monthColor);
  glRotatef (90, 1, 0, 0);
  glTranslatef (menuwidth / 2. + cylssize * 1.1, 0., 0.);
  gluCylinder (quadObj, cylssize, cylssize, (NCYL + 1) * cylssize * 2.6, 10,
               1);
  glTranslatef (-menuwidth - cylssize * 1.1 * 2., 0., 0.);
  gluCylinder (quadObj, cylssize, cylssize, (NCYL + 1) * cylssize * 2.6, 10,
               1);
  glPopMatrix ();
  glPushMatrix ();
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
  for (i = 1; i <= NCYL; i++)
    {
      glTranslatef (0., -cylssize * 2.5, 0.);
      if (cylstext[i - 1] != NULL)
        glCallList (MENU + i);
    }
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  glPopMatrix ();
  glPopMatrix ();
  glShadeModel (GL_FLAT);
  glEndList ();

/* set the loop indice for the creation of all the cylinders lists */
  i = 1;

remake:;
  if (cylstext[i - 1] == NULL)
    return;
/* create the cylinder list */
  glNewList (MENU + i, GL_COMPILE);
  glPushMatrix ();
  glTranslatef (menuwidth / 2., 0., 0.);
  glRotatef (-90, 0, 1, 0);
  glRotatef (-cyls[i - 1][0], 0, 0, 1);
  cylcol[0] = cyls[i][1];
  cylcol[1] = cyls[i][2];
  cylcol[2] = cyls[i][3];
  cylcol[3] = 1.;
  glMaterialfv (GL_FRONT, GL_DIFFUSE, cylcol);
  glEnable (GL_TEXTURE_2D);
  glBindTexture (GL_TEXTURE_2D, texsid[14]);
  gluQuadricTexture (quadObj, GL_TRUE);
  glLoadName (i);
  gluCylinder (quadObj, cylssize, cylssize, menuwidth, 20, 1);
  glLoadName (0);
  gluQuadricTexture (quadObj, GL_FALSE);
  glDisable (GL_TEXTURE_2D);
  glPopMatrix ();
  if (abs (cyls[i - 1][0]) < 60.)
    {
      textposition = 1;
      textmaxwidth = 2300.;
      glPushMatrix ();
      glRotatef (cyls[i - 1][0], 1, 0, 0);
      glTranslatef (0., -cylssize * .5, cylssize * 0.7);
      glDisable (GL_DEPTH_TEST);
      glEnable (GL_LINE_SMOOTH);
      glEnable (GL_BLEND);
      glDisable (GL_LIGHTING);
      glColor4f (0.0, 0.0, 0.0, 1.);
      output (000., 0000., 1.0, 0.4, cylstext[i - 1]);
      glEnable (GL_LIGHTING);
      glDisable (GL_BLEND);
      glDisable (GL_LINE_SMOOTH);
      glEnable (GL_DEPTH_TEST);
      glPopMatrix ();
    }
  glEndList ();

/* if we are initializing the whole menu, loop on each cylinder */
  if (listnb == 0 && i != NCYL)
    {
      i++;
      if (cylstext[i - 1] == NULL)
        i++;
      goto remake;
    }
}

/* redraw the menu WITHOUT the background (lobster etc.) */

void
redrawmenu (void)
{

  glDepthFunc (GL_ALWAYS);
  glDepthMask (GL_FALSE);
  if (currentmenu == RULES || currentmenu == FINDS ||
      (currentmenu == SCORES && object < 5))
    {
      GLdouble planeEqn[] = { 0.000, 1.000, 0.0, 0.0 };  /* clipping plane equation */
      glClipPlane (GL_CLIP_PLANE0, planeEqn);   /* define clipping  plane */
      glEnable (GL_CLIP_PLANE0); /* and enable it */
    }
  glPushMatrix ();
  if (currentmenu == SCORES && object > 4)
    {
      glDisable (GL_CLIP_PLANE0);
      glDepthMask (GL_TRUE);
      glDepthFunc (GL_LESS);
      redraw ();
    }
  else
    {
      glCallList (MENU);
      glutSwapBuffers ();
    }
  glPopMatrix ();
  glDisable (GL_CLIP_PLANE0);
  glDepthMask (GL_TRUE);
  glDepthFunc (GL_LESS);
}

/* draw what needed by the score interface */

void
drawscores (void)
{
  int i;
  char buf[80];

  glColor4f (0.0, 0.0, 1.0, 0.3);
  output (750., 200., 1.0, 1.0, name);
  textmaxwidth = 1400;
  if (validatenextredisplay == 1)
    {
      validatename ();
    }
  else
    {
      output (750., 1200., 0.5, 0.5, text[uselangnb][28]);
      if (strlen (names[0]) != 0)
        output (750., 1100., 0.5, 0.5, text[uselangnb][29]);
      for (i = 0; i < MAXNAMES && strlen (names[i]) != 0; i++)
        {
          sprintf (buf, "%s%s %d", names[i], text[uselangnb][31], games[i]);
          output (750., 1000. - (700. / MAXNAMES) * i, 0.3, 0.3, buf);
        }
    }
}

/* print the text of the scores from the menu */

void
seescores (void)
{
  if (namesee == -1)
    return;
  char buf[80];
  char buft[200];
  char bufs[200];
  int i, j, pos, place;
  long unsigned int tmpval, toprint;

  textmaxwidth = 1400.;
  textmaxsize = 0;
  textposition = 1;
  glColor4f (0.0, 0.2, 0.2, 1.0);
  sprintf (buf, "%s%s %d", names[namesee], text[uselangnb][31],
           games[namesee]);
  output (750., 650., 0.5, 0.5, buf);
  pos = toprint = 0;
  for (i = 0; i < 10; i++)
    {
      if (times[namesee][i] == 0)
        continue;
      for (j = 0, tmpval = ULONG_MAX, place = 0; j < 10; j++)
        {
          if (times[namesee][j] == 0)
            continue;
          if (times[namesee][j] < tmpval && times[namesee][j] > toprint)
            {
              tmpval = times[namesee][j];
              place = j;
            }
        }
      toprint = tmpval;
      if (i != 0)
        sprintf (&buft[pos], "; ");
      if (i != 0)
        pos += sprintf (&bufs[pos], "; ");
      pos +=
        sprintf (&bufs[pos], "%*lu",
                 sprintf (&buft[pos], "%.2f", toprint / 1000.),
                 steps[namesee][place]);
    }
  output (750., 500., 0.4, 0.3, text[uselangnb][16]);
  output (750., 400., 0.5, 0.3, buft);
  output (750., 300., 0.4, 0.3, text[uselangnb][17]);
  output (750., 200., 0.5, 0.3, bufs);
}

/* add the directory prefix before calling the function to load textures */

void
dLoadRGBMipmaps (const char *imageFile, GLint intFormat)
{
  char tmp[250] = SKINDIR "/";

  strcat (tmp, skin[useskinnb]);
  strcat (tmp, ".jsk/");
  strcat (tmp, imageFile);
  LoadRGBMipmaps (tmp, intFormat);
}

/* read the textures */

void
loadtextures (int first /* says if it is the first time */ )
{
  glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

  if (first)
    glGenTextures (1, &texsid[0]);
  glBindTexture (GL_TEXTURE_2D, texsid[0]);
  dLoadRGBMipmaps ("ground.rgb", 4);
  glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                   GL_LINEAR_MIPMAP_LINEAR);
  glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  //glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);

  if (first)
    glGenTextures (1, &texsid[1]);
  glBindTexture (GL_TEXTURE_2D, texsid[1]);
  dLoadRGBMipmaps ("ext.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[2]);
  glBindTexture (GL_TEXTURE_2D, texsid[2]);
  dLoadRGBMipmaps ("wall1.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[3]);
  glBindTexture (GL_TEXTURE_2D, texsid[3]);
  dLoadRGBMipmaps ("wall2.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[4]);
  glBindTexture (GL_TEXTURE_2D, texsid[4]);
  dLoadRGBMipmaps ("wall3.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[5]);
  glBindTexture (GL_TEXTURE_2D, texsid[5]);
  dLoadRGBMipmaps ("wall4.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[6]);
  glBindTexture (GL_TEXTURE_2D, texsid[6]);
  dLoadRGBMipmaps ("wall5.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[7]);
  glBindTexture (GL_TEXTURE_2D, texsid[7]);
  dLoadRGBMipmaps ("wall6.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[8]);
  glBindTexture (GL_TEXTURE_2D, texsid[8]);
  dLoadRGBMipmaps ("wall7.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[9]);
  glBindTexture (GL_TEXTURE_2D, texsid[9]);
  dLoadRGBMipmaps ("wall8.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[10]);
  glBindTexture (GL_TEXTURE_2D, texsid[10]);
  dLoadRGBMipmaps ("wall9.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[11]);
  glBindTexture (GL_TEXTURE_2D, texsid[11]);
  dLoadRGBMipmaps ("wall10.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[12]);
  glBindTexture (GL_TEXTURE_2D, texsid[12]);
  dLoadRGBMipmaps ("cloud.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[13]);
  glBindTexture (GL_TEXTURE_2D, texsid[13]);
  dLoadRGBMipmaps ("logo.rgb", 4);

  if (first)
    glGenTextures (1, &texsid[14]);
  glBindTexture (GL_TEXTURE_2D, texsid[14]);
  glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  dLoadRGBMipmaps ("menu.rgb", 4);
}

/* create the lists to call for the environment */
void
createterrain (void)
{
  int number;

  glNewList (wallmax, GL_COMPILE);
  glBindTexture (GL_TEXTURE_2D, texsid[1]);
  glBegin (GL_QUAD_STRIP);
  glTexCoord2f (T_Y + T_X, 0.0);
  glVertex3f (+T_C / 2., 0.0, -T_C / 2.);
  glTexCoord2f (T_Y + T_X, 1.0);
  glVertex3f (+T_C / 2., T_H * 2., -T_C / 2.);
  glTexCoord2f (T_Y + T_X / 2., 0.0);
  glVertex3f (-T_C * T_X + T_C / 2., 0.0, -T_C / 2.);
  glTexCoord2f (T_Y + T_X / 2., 1.0);
  glVertex3f (-T_C * T_X + T_C / 2., T_H * 2., -T_C / 2.);
  glTexCoord2f (T_Y / 2. + T_X / 2., 0.0);
  glVertex3f (-T_C * T_X + T_C / 2., 0.0, -T_C / 2. + T_Y * T_C);
  glTexCoord2f (T_Y / 2. + T_X / 2., 1.0);
  glVertex3f (-T_C * T_X + T_C / 2., T_H * 2., -T_C / 2. + T_Y * T_C);
  glTexCoord2f (T_Y / 2., 0.0);
  glVertex3f (+T_C / 2., 0.0, -T_C / 2. + T_Y * T_C);
  glTexCoord2f (T_Y / 2., 1.0);
  glVertex3f (+T_C / 2., T_H * 2., -T_C / 2. + T_Y * T_C);
  glTexCoord2f (0.0, 0.0);
  glVertex3f (+T_C / 2., 0.0, -T_C / 2.);
  glTexCoord2f (0.0, 1.0);
  glVertex3f (+T_C / 2., T_H * 2., -T_C / 2);
  glEnd ();
  glEndList ();

  glNewList (TOPBOT, GL_COMPILE);

  glShadeModel (GL_SMOOTH);
  glBegin (GL_QUADS);
  glColor4f (ceilingColor[0] / 1.1, ceilingColor[1], ceilingColor[2],
             ceilingColor[3]);
  glVertex3f (+T_C / 2.0, T_H * 2., -T_C / 2.0);
  glColor4f (ceilingColor[0], ceilingColor[1] / 1.1, ceilingColor[2],
             ceilingColor[3]);
  glVertex3f (+T_C / 2.0, T_H * 2., T_Y * T_C - T_C / 2.0);
  glColor4f (ceilingColor[0], ceilingColor[1], ceilingColor[2],
             ceilingColor[3]);
  glVertex3f (-T_X * T_C + T_C / 2.0, T_H * 2., T_Y * T_C - T_C / 2.0);
  glColor4f (ceilingColor[0] / 1.1, ceilingColor[1] / 1.1, ceilingColor[2],
             ceilingColor[3]);
  glVertex3f (-T_X * T_C + T_C / 2.0, T_H * 2., -T_C / 2.0);
  glEnd ();
  glShadeModel (GL_FLAT);

  glEnable (GL_TEXTURE_2D);
  glBindTexture (GL_TEXTURE_2D, texsid[0]);

  glBegin (GL_QUADS);
  glTexCoord2f (0.0, 0.0);
  glVertex3f (+T_C / 2.0, 0.0, -T_C / 2.0);
  glTexCoord2f (0.0, 16.0);
  glVertex3f (-T_X * T_C + T_C / 2.0, 0.0, -T_C / 2.0);
  glTexCoord2f (16.0, 16.0);
  glVertex3f (-T_X * T_C + T_C / 2.0, 0.0, T_Y * T_C - T_C / 2.0);
  glTexCoord2f (16, 0.0);
  glVertex3f (+T_C / 2.0, 0.0, T_Y * T_C - T_C / 2.0);
  glEnd ();

  glDisable (GL_TEXTURE_2D);

  glEndList ();

  glNewList (CLOUDS, GL_COMPILE);

  glEnable (GL_BLEND);
  glBindTexture (GL_TEXTURE_2D, texsid[12]);
  for (number = 20; number > 0; number--)
    {
      glPushMatrix ();
      glTranslatef (-(myrand () - 0.25) * T_C * T_X * 2.,
                    number * 100 + T_H * 4.,
                    (myrand () - 0.25) * T_C * T_Y * 2.);
      glRotatef (180, 1, 0, 0);
      glRotatef (myrand () * 360., 0, 1, 0);
      glScalef ((myrand () + 0.1) * 4., (myrand () + 0.1) * 4.,
                (myrand () + 0.1) * 4.);
      glBegin (GL_QUADS);
      glTexCoord2f (0.0, 0.0);
      glVertex3f (+T_C, 0.0, -T_C);
      glTexCoord2f (0.0, 1.0);
      glVertex3f (-T_C, 0.0, -T_C);
      glTexCoord2f (1.0, 1.0);
      glVertex3f (-T_C, 0.0, +T_C);
      glTexCoord2f (1, 0.0);
      glVertex3f (+T_C, 0.0, +T_C);
      glEnd ();
      glPopMatrix ();
    }
  glDisable (GL_BLEND);

  glEndList ();

}


/* set all what must be to launch the game -- graphic part */

void
setgameattribs (void)
{
  GLfloat black[] = { 0., 0., 0., 1. };
  GLfloat texColor[] = { 1.0, 1.0, 1.0, 1.0 };

//glClearColor(fogColor[0], fogColor[1], fogColor[2], fogColor[3]);
//glutIgnoreKeyRepeat(1);
  glutDisplayFunc (redraw);
  glutReshapeFunc (reshape);
  glutMouseFunc (mouse);
#if (MYFREEGLUT)
  glutMouseWheelFunc (mouse);
#endif
  glutKeyboardFunc (KeyDown);
  glutSpecialFunc (SKeyDown);
  glutSpecialUpFunc (KeyUp);
  glutMotionFunc (motionactive);
  glutPassiveMotionFunc (motionpassive);
  glutSetCursor (GLUT_CURSOR_NONE);

  makepince ();
  makearms ();
  makeant ();
  maketail ();
  makebody ();
  makelegs ();

  makehead ();
  makemsoccer ();

  createsmap ();

  glVertexPointer (3, GL_FLOAT, sizeof (GLfloat) * 5, &wallsdata[2]);
  glTexCoordPointer (2, GL_FLOAT, sizeof (GLfloat) * 5, &wallsdata[0]);

#if USEGLEW
#  ifdef GL_ARB_vertex_buffer_object
  if (GLEW_ARB_vertex_buffer_object)
    {
      glGenBuffersARB (1, &vbobuf);
      glBindBufferARB (GL_ARRAY_BUFFER, vbobuf);
      glBufferDataARB (GL_ARRAY_BUFFER, 490 * sizeof *wallsdata, NULL,
                       GL_STATIC_DRAW);
      glBufferSubDataARB (GL_ARRAY_BUFFER, 0, 490 * sizeof *wallsdata,
                          wallsdata);

#    define BUFFER_OFFSET(a) ((char *)NULL + (a))
      glVertexPointer (3, GL_FLOAT, sizeof (GLfloat) * 5, BUFFER_OFFSET (8));
      glTexCoordPointer (2, GL_FLOAT, sizeof (GLfloat) * 5,
                         BUFFER_OFFSET (0));
#    undef BUFFER_OFFSET
    }
#  endif
#endif

  glEnableClientState (GL_VERTEX_ARRAY);
  glEnableClientState (GL_TEXTURE_COORD_ARRAY);

  createterrain ();

  glEnable (GL_DEPTH_TEST);
  glEnable (GL_LIGHTING);
  glEnable (GL_CULL_FACE);
  glShadeModel (GL_FLAT);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glLightModelfv (GL_LIGHT_MODEL_AMBIENT, black);
  glLightfv (GL_LIGHT1, GL_DIFFUSE, lightColor);
  glLightf (GL_LIGHT1, GL_CONSTANT_ATTENUATION, 0.1);
  glLightf (GL_LIGHT1, GL_LINEAR_ATTENUATION, 0.01);
  glLightfv (GL_LIGHT0, GL_POSITION, lightPosition);
  glEnable (GL_LIGHT0);
  glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glHint (GL_LINE_SMOOTH_HINT, GL_NICEST);
  glHint (GL_POINT_SMOOTH_HINT, GL_NICEST);
  glHint (GL_POLYGON_SMOOTH_HINT, GL_NICEST);
  glTexEnvfv (GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, texColor);
//static float            fog_density = 0.001*2;
//glHint(GL_FOG_HINT, GL_NICEST);
//glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST );
//glFogi(GL_FOG_MODE, GL_LINEAR);
//glFogi(GL_FOG_END, 2000);
//glFogf(GL_FOG_DENSITY, fog_density);
//glFogfv(GL_FOG_COLOR, fogColor);
//glEnable(GL_FOG);

  glSelectBuffer (SELBUFFLEN, selbuff);
}
