/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include <GL/glut.h>

#include "core.h"
#include "display.h"
#include "draw.h"
#include "interface.h"
#include "jomar3D.h"
#include "scores.h"
#include "sound.h"
#include "utils.h"


static int rearturn = 0;        // whether we rotate the cyls backwards
static int uppressed;           // remember if the up key is pressed
static int shading, moving, beginx, beginy;

static void domenu (void);
static void menurot (int dummy);




/* stop the crazy speed of the boost */
void
stopboost (int dummy)
{
  (void) dummy;
  gotime = 2. * 1000;
#if (USESOUNDSDLMIXER)
  if (playsound)
    Mix_HaltChannel (BOOST);
#endif
}

/* process the user's keyboard actions */
void
KeyDown (unsigned char c, int x, int y)
{
  (void) x, (void) y;
  int i;

  if (currentmenu != NOMENU && isdigit (c))
    {                           // emulate that pressed number keys are as if an object was selected in the menu
      object = atoi ((char *) &c);
      domenu ();
      return;
    }

/*  the first letter of each cylinder in the menu must enter this item - and stop all other action */
  if (currentmenu != NOMENU)
    {
      for (i = 0; cylstext[i] != NULL && i < NCYL - 1; i++)
        {
          if (c == tolower (cylstext[i][0]) || c == toupper (cylstext[i][0]))
            {
#if (DEBUG)
              printf ("cylinder number %d met by key presssed\n", i + 1);
#endif
              object = i + 1;
              domenu ();
              return;
            }
        }
    }

  switch (c)
    {

    case 27:                   /* Escape */
      exit (0);
      break;
    case 'm':                  /* return into main menu */
      changemenu (MP);
      break;
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
    case 'c':                  /* change music */
      changemusic ();
      break;
#endif
    case 'i':
      introstarttime = 0;
      glutPostRedisplay ();
      break;
    case 'w':
      glutSwapBuffers ();
      break;
    case 'v':                  /* reposition view */
    case '1':
      anglex = 0.;
      angley = 0.;
      posex = 0;
      posey = -T_C * 1.5 + 1.;
      glutPostRedisplay ();
      break;
    case '2':
      anglex = 0.;
      angley = 0.;
      posex = 0;
      posey = -T_C / 2.;
      glutPostRedisplay ();
      break;
    case '3':
      anglex = -45.;
      angley = 0.;
      posex = 0;
      posey = -T_C / 2.;
      glutPostRedisplay ();
      break;
    case '4':
      anglex = 45.;
      angley = 0.;
      posex = 0;
      posey = -T_C / 2.;
      glutPostRedisplay ();
      break;
    case '5':
      anglex = 180.;
      angley = 0.;
      posex = 0;
      posey = T_C / 2.;
      glutPostRedisplay ();
      break;
    case 's':                  /* toggle game mode (full screen) */
    case 'S':
    case 'f':
    case 'F':
      g_winmode = !g_winmode;
      windowmanage ();
#if (MYFREEGLUT == 0)
      setgameattribs ();        //  only if a new context is used
      if (currentmenu != NOMENU)
        createmenulists (0);    // idem
#endif
      break;
    case 'a':                  //  trying to eat
    case 'A':
    case 13:
      if (gotime != 0.75 * 1000.)
        tryeat ();              // only if last move we didn't try to eat
      break;
    case 'b':                  //  boosting the lobster's speed
    case 'B':
      if (boostrac < 3)
        {
          boostrac += 1;
          gotime = 99.;
          maketail ();
          glutPostRedisplay ();
          glutTimerFunc (10. * 1000, stopboost, 0);
#if (USESOUNDSDLMIXER)
          mixsound (BOOST);
#endif
        }
      break;
    default:
#if (DEBUG)
      printf ("unused key %c (code %d)\n", c, c);
#endif
      break;
    }
}
void
SKeyDown (int c, int x, int y)
{
  (void) x, (void) y;
  switch (c)
    {
    case GLUT_KEY_RIGHT:
      if (!going)
        {
          if (tryplace (2) == 1)
            {
              action = 2;
            }
          else
            {
              action = 0;
            }
        }
      else
        {
          nextaction = c;
        }
      break;
    case GLUT_KEY_DOWN:
      if (!going)
        {
          if (tryplace (4) == 1)
            {
              action = 4;
            }
          else
            {
              action = 0;
            }
        }
      else
        {
          nextaction = c;
        }
      break;
    case GLUT_KEY_LEFT:
      if (!going)
        {
          if (tryplace (3) == 1)
            {
              action = 3;
            }
          else
            {
              action = 0;
            }
        }
      else
        {
          nextaction = c;
        }
      break;
    case GLUT_KEY_UP:
      if (!going)
        {
          if (tryplace (1) == 1)
            {
              action = 1;
            }
          else
            {
              action = 0;
            }
        }
      else
        {
          nextaction = c;
        }
      uppressed = 1;
      break;
    case GLUT_KEY_F11:
      KeyDown ('s', 0, 0);
      return;
      break;
    case GLUT_KEY_F10:
      KeyDown ('m', 0, 0);
      return;
      break;
    default:
#if (DEBUG)
      printf ("%c %d\n", c, c);
#endif
      break;
    }

/* start the motion if necessary */

  if (c == GLUT_KEY_RIGHT ||
      c == GLUT_KEY_LEFT || c == GLUT_KEY_UP || c == GLUT_KEY_DOWN)
    {
      nextaction = c;

      if (!going && action != 0)
        {
          if (gotime == 100. && godist != T_C * 2.)
            {                   //slow if we had tryed to eat
              gotime = 0.75 * 1000.;
            }
          else if (gotime != 99.)
            {
              if (action == 3 || action == 2)
                gotime = 0.5 * 1000.;
              if (action == 1 || action == 4)
                if (godist != T_C * 2.)
                  gotime = 0.25 * 1000.;
            }
          going = 1;
#if (USESOUNDSDLMIXER)
          if (playsound)
            if (Mix_Playing (WALK) == 0 && gotime != 100.)
              mixsound (WALK);
#endif
          tbase = glutGet (GLUT_ELAPSED_TIME);
          if (action == 1 || action == 2 || action == 4)
            anglehom = anglebasehom = (terrain[homposx][homposy] - 1) * 90.0;
          if (action == 3)
            anglehom = anglebasehom = -(terrain[homposx][homposy] - 1) * 90.0;
          glutIdleFunc (forcedraw);
          if (lastaction == 0)
            glutTimerFunc (1000, nocur, 0);
        }
      lastaction = action;
      if (action == 0)
        {                       // don't stop behind a martian, retry later
          if (!tryplace (1))
            startbehindmart (0);
        }
    }
}
void
KeyUp (int c, int x, int y)
{
  (void) x, (void) y;
  switch (c)
    {
    case GLUT_KEY_UP:
      uppressed = 0;
    case GLUT_KEY_RIGHT:
    case GLUT_KEY_LEFT:
    case GLUT_KEY_DOWN:
      if (nextaction == c)
        nextaction = 0;
    default:
      break;
    }
  if (uppressed && !nextaction)
    SKeyDown (GLUT_KEY_UP, 1, 1);   //  restart if we had turn
}

/* manipulate the menus */

void
changemenu (int menu)
{
  object = 0;
  if (currentmenu == NOMENU)
    initgame ();
  currentmenu = menu;
  createmenulists (0);
  redraw ();                    //   resolves a lot of problems with menus when there are two buffers AND one for the window
  redraw ();                    //                                 (Windows)
  glutPostRedisplay ();
}
void loadscores (void);
void
domenu (void)
{
  if (currentmenu == MP && object == 1)
    initgame ();
  if (currentmenu == MP && object == 2)
    changemenu (RULES);
  if (currentmenu == MP && object == 3)
    {
      loadscores ();
      namesee = -1;
      changemenu (SCORES);
    }
  if (currentmenu == MP && object == 4)
    changemenu (OPTIONS);
  if (currentmenu == MP && object == 5)
    changemenu (FINDS);
  if (currentmenu == RULES && object == 1)
    changemenu (MP);
  if (currentmenu == SCORES)
    {
      if (object == 10)
        {
          changemenu (MP);
          return;
        }
      if (cylstext[object] == NULL)
        changemenu (MP);
      namesee = object - 1;
      redraw ();
      glutPostRedisplay ();
    }
  if (currentmenu == OPTIONS && object == 4)
    changemenu (MP);
  if (currentmenu == OPTIONS && (object == 1 || object == 2 || object == 3))
    menurot (0);
  if (currentmenu == FINDS && object == 1)
    changemenu (MP);
}

/* process the mouse's actions */

void
mouse (int button, int state, int x, int y)
{
  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
    {
      shading = 1;
      beginx = x;
      beginy = y;
    }
  if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
    {
      shading = 0;
    }

  if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
    {
      moving = 1;
      beginx = x;
      beginy = y;
    }
  if (button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
    {
      moving = 0;
    }

  if (state == GLUT_DOWN && object != 0 && button != GLUT_MIDDLE_BUTTON)
    {
      if ((button > 2 && cyls[object - 1][0] == 0.) || button < 3)
        domenu ();
      if (button == GLUT_RIGHT_BUTTON || button == 3)
        rearturn = 1;
    }

  if (button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN)
    {
      openp = 1;
      glutPostRedisplay ();
      tryeat ();
    }
  if (button == GLUT_MIDDLE_BUTTON && state == GLUT_UP)
    {
      openp = 0;
      glutPostRedisplay ();
    }

}

void
motionactive (int x, int y)
{
  glutSetCursor (GLUT_CURSOR_CYCLE);
  if (shading)
    {
      anglex = anglex + (x - beginx) / 2.;
      angley = angley + (y - beginy) / 2.;
    }
  if (moving)
    {
      float dx = (x - beginx) / 4., dy = (y - beginy) / 4., dx2, dy2;

      dx2 = sin (anglex * degrad) * dy;
      dy2 = cos (anglex * degrad) * dy;
      dx2 += sin (anglex * degrad + M_PI / 2.) * dx;
      dy2 += cos (anglex * degrad + M_PI / 2.) * dx;
      if (posex + dx2 < T_C / 2. - 1. && posex + dx2 > -T_C / 2. + 1. &&
          posey + dy2 < T_C / 2. - 1. && posey + dy2 > -T_C * 1.5 + 1.)
        {
          posex += dx2;
          posey += dy2;
        }
    }
  beginx = x;
  beginy = y;
  glutPostRedisplay ();
}

/* change an option from the menu */
void
changeopt (int opt)
{
#if (DEBUG)
  printf ("Changing option number %d\n", opt);
#endif
  if (opt == 0)
    {
    poor:uselangnb += 1;
      if (rearturn)
        uselangnb -= 2;
      if (uselangnb >= nlang)
        uselangnb = 0;
      if (uselangnb < 0)
        uselangnb = nlang - 1;
      if (strcmp (lang[uselangnb], "unavailable") == 0)
        goto poor;
      createmenulists (0);
      redraw ();
      glutPostRedisplay ();
    }
  if (opt == 1)
    {
      useskinnb += 1;
      if (rearturn)
        useskinnb -= 2;
      if (useskinnb >= nskin)
        useskinnb = 0;
      if (useskinnb < 0)
        useskinnb = nskin - 1;
      createmenulists (0);
      loadtextures (0);
      redraw ();
      glutPostRedisplay ();
    }
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
  if (opt == 2)
    {
      playsound = !playsound;
#  if (USESOUNDSDLMIXER)
      if (playsound)
        Mix_Resume (-1);
      else
        Mix_Pause (-1);
#  endif
      createmenulists (0);
      redraw ();
      glutPostRedisplay ();
    }
#endif
  saveoptions ();
}

/* rotate a cylinder from the menu (changing an option) */

void
menurot (int dummy)
{
  static int beginrot = 0;
  static int cylnb = -1;
  static int dochange = 1;
  float rottime = 1000.;

  (void) dummy;
  if (beginrot == 0)
    beginrot = glutGet (GLUT_ELAPSED_TIME) - 1; // -1: so the the cylinder moves already now (don't accept more event)
  float elapsed = glutGet (GLUT_ELAPSED_TIME) - beginrot;

  if (cylnb == -1)
    cylnb = object - 1;
  if (elapsed > rottime)
    {
      cyls[cylnb][0] = 0.;
      beginrot = 0;
      cylnb = -1;
      dochange = 1;
      rearturn = 0;
      goto draw;
    }
  cyls[cylnb][0] = elapsed / rottime * 2. * 90.;
  glutTimerFunc (50., menurot, 0);
  if (elapsed > rottime / 2.)
    {
      cyls[cylnb][0] -= 180.;
      if (dochange)
        {
          cyls[cylnb][0] = -90.; // flag so we don't draw a false cylinder when remaking the menu
          changeopt (cylnb);
          dochange = 0;
          return;
        }
    }
  if (rearturn)
    cyls[cylnb][0] = -cyls[cylnb][0];
#if (DEBUG)
  printf ("degree of this cylinder %f\n", cyls[cylnb][0]);
#endif
draw:;
  createmenulists (cylnb + 1);
  redrawmenu ();
}

/* change the color of a cylinder (the mouse is on this cylinder) */

void
menucol (int sobject)
{
  int col;

  if (currentmenu == NOMENU)
    return;
  if (object == 0)
    {
      redrawmenu ();
      return;
    }
  if (object != sobject)
    return;
new:col = myrand () * (3);
  if (col == 3)
    goto new;                   // to have a good part on 0,1,2 it must be few 3, skip
  cyls[object][col + 1] = myrand ();
  createmenulists (object);

  redrawmenu ();

/* reinitialize the color so it is composed with 2 times 1; black and dark colors are unavailable */
  cyls[object][1] = cyls[object][2] = cyls[object][3] = 1.0;
  createmenulists (object);

/* cause the next redisplay */
  glutTimerFunc (50., menucol, object);
}
void setview (void);

/* when in the menu, do a special intra-rendering to know if a volume (cylinder) is under the pointer */

void
motionpassive (int x, int y)
{
  GLint viewport[4];
  GLint hits;
  GLuint depth = (GLuint) ~ 0;
  unsigned int getThisName;
  GLuint names, *ptr;
  GLint i;
  int lastobject = object;

  glutSetCursor (GLUT_CURSOR_INFO);

  object = 0;
  if (currentmenu == NOMENU)
    goto end;

  /* fill in the current viewport parameters */
  viewport[0] = 0;
  viewport[1] = 0;
  viewport[2] = winwidth;
  viewport[3] = winheight;

  glRenderMode (GL_SELECT);
  glInitNames ();
  glPushName (-1);

  /* setup a picking matrix and render into selection buffer */
  glMatrixMode (GL_PROJECTION);
  glPushMatrix ();
  glMatrixMode (GL_MODELVIEW);
  glPushMatrix ();

  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluPickMatrix (x, winheight - y, 1.0, 1.0, viewport);
  setview ();

  glMatrixMode (GL_MODELVIEW);
  glCallList (MENU);

  glMatrixMode (GL_PROJECTION);
  glPopMatrix ();
  glMatrixMode (GL_MODELVIEW);
  glPopMatrix ();
  hits = glRenderMode (GL_RENDER);

  if (hits == 0)
    goto end;

/* process results of selecting */
  ptr = (GLuint *) selbuff;
  for (i = 0; i < hits; i++)
    {                           /* for each hit  */
      getThisName = 0;
      names = *ptr;
      ptr++;                    /* skip # name */
      if (*ptr <= depth)
        {
          depth = *ptr;
          getThisName = 1;
        }
      ptr++;                    /* skip z1 */
      if (*ptr <= depth)
        {
          depth = *ptr;
          getThisName = 1;
        }
      ptr++;                    /* skip z2 */

      if (getThisName)
        object = *ptr;
      ptr += names;             /* skip the names list */
    }
#if (DEBUG)
  printf ("Selected object : %d\n", object);
#endif
  if (object == -1)
    {
      object = 0;
      goto end;
    }

  if (object != lastobject)
    {
      menucol (object);
#if (USESOUNDSDL) || (USESOUNDSDLMIXER)
      glutTimerFunc (10, mixsound, SMENU1 + object - 1);
#endif
      glutPostRedisplay ();
    }
end:;
}
